/****************************************************************************
* NVMC                                                                      *
* Visual and Computer Graphics Library                                      *
*                                                                           *
* Copyright(C) 2007                                                         *
* ISTI - Italian National Research Council                                  *
*                                                                           *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *   
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/


#include <SDL.h>
#include <GL/glew.h>
#include <math.h>
#include <time.h>
#include <vcg/space/point3.h>
#include <vector>
#include <GL/glut.h>
#include <GL/glu.h>

#include <iostream>

#include <circuit.h>
#include <street.h>

//include x caricamento modello mesh .ply
#include <vcg/simplex/vertexplus/base.h>
#include <vcg/simplex/faceplus/base.h>
#include <vcg/complex/trimesh/base.h>
#include <vcg/complex/trimesh/update/bounding.h>
#include <vcg/complex/trimesh/update/normal.h>
#include <wrap/io_trimesh/import.h>
#include <wrap/gl/trimesh.h>


#include "../client/game_client.h"
#include <wrap/gui/trackball.h>
#include "game_viewer.h"
#include "race.h"

#include "tga.h"


using namespace vcg;
using namespace std;

class CEdge;    // dummy prototype never used
class CFace;


class CVertex : public VertexSimp2< CVertex, CEdge, CFace, vert::Coord3f, vert::Normal3f >{};
//cosi non vedo i colori della macchina
//class CFace   : public FaceSimp2< CVertex, CEdge, CFace, face::VertexRef, face::Normal3f > {};

//cosi vedo i colori della macchina
class CFace   : public FaceSimp2< CVertex, CEdge, CFace, face::VertexRef, face::Normal3f , face::Color4b  > {}; 

/*the mesh is a container of vertices and a container of faces */ 
class CMesh   : public vcg::tri::TriMesh< vector<CVertex>, vector<CFace> > {};
CMesh mesh; //car mesh

vcg::GlTrimesh<CMesh> glWrap;
//vcg::Trackball track;


// VARIABILI PER IL PUNTO DI VISTA
vcg::Point3f pov;
bool free_look=false;
vcg::Point3f free_pov;
vcg::Point3f free_position;
float alpha=0,tetha=0;
const int norma=1;
const int passo=50;
int x,y,z;

/** Client  */
extern  NvmcClient cli;

/* value of pi */
#define M_PI       3.14159265358979323846

/** viewport size */
int width_wps,height_wps;

/* valori per il lighting */
GLfloat mat_lig[32];






//TEXTURES
#define TexRoadFile     "road.bmp"	//path texture strada
#define TexGrassFile    "grass.bmp" //path texture erba
#define TexTreeFile     "tree.tga" //path alberello
#define TexBuildingFile "build.bmp" //path palazzo

//path dello skybox
#define TexCubeXposFile "mount_posx.bmp"
#define TexCubeXnegFile "mount_negx.bmp"
#define TexCubeYposFile "mount_posy.bmp"
#define TexCubeYnegFile "mount_negy.bmp"
#define TexCubeZposFile "mount_posz.bmp"
#define TexCubeZnegFile "mount_negz.bmp"


#define TexFarifile "headlight2.bmp"
#define TexTunnelFile "tunnel_1.bmp"
#define TexCarFile "text_car4.bmp"


//indice delle textures
#define TexRoad   0	
#define TexGrass  1
#define TexTree	  2
#define TexBuilding 3
#define TexSkyBoxXpos 4
#define TexSkyBoxXneg 5
#define TexSkyBoxYpos 6
#define TexSkyBoxYneg 7
#define TexSkyBoxZpos 8
#define TexSkyBoxZneg 9
#define TexFari 10
#define Textunnel 11
#define TexCar 12

#define TexSkyBoxXposDin 13
#define TexSkyBoxXnegDin 14
#define TexSkyBoxYposDin 15
#define TexSkyBoxYnegDin 16
#define TexSkyBoxZposDin 17
#define TexSkyBoxZnegDin 18

//numero totale delle texture
#define NUM_TEXTURES  19 

//vettore di tutte le texture
GLuint texName[NUM_TEXTURES];


// MODELLO TRIDIMENSIONALE AUTO 
#define CAR_MODEL_PATH "z3.ply" //path del modello dell'auto


// PATH DEL CIRCUITO
char *circuito="circuit1.nvc";

//IN TUNNEL
bool NelTunnel;

//MOTION BLUR
#define MAX_RENDER 10
int render;
static bool motion_on=false;

//funzione per la gestione della vista libera
vcg::Point3f free_view(vcg::Point3f free_pov,vcg::Point3f free_position){
		free_position.X()=free_pov.X()+norma*cos(alpha)*cos(tetha);
		free_position.Y()=free_pov.Y()+norma*cos(alpha)*sin(tetha);
		free_position.Z()=free_pov.Z()+norma*sin(alpha)*cos(tetha);
		return free_position;
}

//
// INIZIALIZZAZIONE DELLE TEXTURES 
//funzione che carica le textures(type 0 no canale alfa, 1 canale alfa)
void loadTexture(const char* fname, int id, int type){
	SDL_Surface *s = SDL_LoadBMP(fname);

	glBindTexture(GL_TEXTURE_2D,id);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	
	if(type==0){
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		/*definiamo come deve essere fatto il smapling nel caso di MAGNIFICATION e
		di MINIFICATION */
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);

		// MIPMAPPING
		gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB, s->w, s->h, GL_BGR, GL_UNSIGNED_BYTE, s->pixels);
	} else {
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		// definizione del filtering della texture
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, s->w, s->h,	0, GL_BGRA_EXT, GL_UNSIGNED_BYTE, s->pixels);
	}

	SDL_FreeSurface(s);
}

void initTexture(void)
{
bool wrap = true;

 // Generate textures
 glGenTextures(NUM_TEXTURES, texName);

 glPixelStorei(GL_UNPACK_ALIGNMENT,4);

 SDL_Surface* bmpFile;



 // texture x la strada
  bmpFile = SDL_LoadBMP(TexRoadFile);
 glBindTexture( GL_TEXTURE_2D, texName[TexRoad] );
 glTexEnvf( GL_TEXTURE_2D, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 // if wrap is true, the texture wraps over at the edges (repeat)
 //       ... false, the texture ends at the edges (clamp)
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap ? GL_REPEAT : GL_CLAMP );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap ? GL_REPEAT : GL_CLAMP );
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_RGB, GL_UNSIGNED_BYTE, bmpFile->pixels);
 SDL_FreeSurface(bmpFile);


 //texture x l'erba
  bmpFile = SDL_LoadBMP(TexGrassFile);
 glBindTexture( GL_TEXTURE_2D, texName[TexGrass] );
 //glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap ? GL_REPEAT : GL_CLAMP );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap ? GL_REPEAT : GL_CLAMP );
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h,GL_RGB /*GL_RGB*/, GL_UNSIGNED_BYTE, bmpFile->pixels);
 SDL_FreeSurface(bmpFile);

 //texture x l'albero ( in .tga)
 tgaInfo *tree;
 tree = tgaLoad(TexTreeFile);
 glBindTexture(GL_TEXTURE_2D,texName[TexTree]);
 glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_WRAP_S,	GL_CLAMP);
 glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_WRAP_T,	GL_CLAMP);
 glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_MAG_FILTER	,GL_NEAREST);
 glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_MIN_FILTER	,GL_LINEAR);	
 glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, tree->width, tree->height, 0, GL_RGBA, GL_UNSIGNED_BYTE, tree->imageData);

 //texture x il palazzo
 bmpFile = SDL_LoadBMP(TexBuildingFile);
 glBindTexture( GL_TEXTURE_2D, texName[TexBuilding] );
 //glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_RGB, GL_UNSIGNED_BYTE, bmpFile->pixels);
 SDL_FreeSurface(bmpFile);

 // SKYBOX
 //texture per la x positive
 bmpFile = SDL_LoadBMP(TexCubeXposFile);
 glBindTexture( GL_TEXTURE_2D, texName[TexSkyBoxXpos] );
 //glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_RGB, GL_UNSIGNED_BYTE, bmpFile->pixels);
 SDL_FreeSurface(bmpFile);


 //texture per la x negative
 bmpFile = SDL_LoadBMP(TexCubeXnegFile);
 glBindTexture( GL_TEXTURE_2D, texName[TexSkyBoxXneg] );
 //glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_RGB, GL_UNSIGNED_BYTE, bmpFile->pixels);
 SDL_FreeSurface(bmpFile);

 //texture per la y positive
 bmpFile = SDL_LoadBMP(TexCubeYposFile);
 glBindTexture( GL_TEXTURE_2D, texName[TexSkyBoxYpos] );
 //glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_RGB, GL_UNSIGNED_BYTE, bmpFile->pixels);
 SDL_FreeSurface(bmpFile);

  //texture per la y negative
 bmpFile = SDL_LoadBMP(TexCubeYnegFile);
 glBindTexture( GL_TEXTURE_2D, texName[TexSkyBoxYneg] );
 //glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_RGB, GL_UNSIGNED_BYTE, bmpFile->pixels);
 SDL_FreeSurface(bmpFile);

 //texture per la z positive
 bmpFile = SDL_LoadBMP(TexCubeZposFile);
 glBindTexture( GL_TEXTURE_2D, texName[TexSkyBoxZpos] );
 //glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_RGB, GL_UNSIGNED_BYTE, bmpFile->pixels);
 SDL_FreeSurface(bmpFile);

  //texture per la z negative
 bmpFile = SDL_LoadBMP(TexCubeZnegFile);
 glBindTexture( GL_TEXTURE_2D, texName[TexSkyBoxZneg] );
 //glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_RGB, GL_UNSIGNED_BYTE, bmpFile->pixels);
 SDL_FreeSurface(bmpFile);


 
   //texture per i fari
 bmpFile = SDL_LoadBMP(TexFarifile);
 glBindTexture( GL_TEXTURE_2D, texName[TexFari] );
 //glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 // definizione del wrapping delle coordinate
 glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );
 glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );
 // definizione del filtering della texture
 glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
 glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_RGB, GL_UNSIGNED_BYTE, bmpFile->pixels);
 //glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, bmpFile ->w, bmpFile ->h,	0, GL_BGRA_EXT, GL_UNSIGNED_BYTE, bmpFile ->pixels);
 //glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 //glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_BLEND);
 //glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
 glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_CLAMP );
 SDL_FreeSurface(bmpFile);


// texture x i tunnel
  bmpFile = SDL_LoadBMP(TexTunnelFile);
 glBindTexture( GL_TEXTURE_2D, texName[Textunnel] );
 glTexEnvf( GL_TEXTURE_2D, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 // if wrap is true, the texture wraps over at the edges (repeat)
 //       ... false, the texture ends at the edges (clamp)
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap ? GL_REPEAT : GL_CLAMP );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap ? GL_REPEAT : GL_CLAMP );
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_RGB, GL_UNSIGNED_BYTE, bmpFile->pixels);
 SDL_FreeSurface(bmpFile);

// texture x la macchina
  bmpFile = SDL_LoadBMP(TexCarFile);
 glBindTexture( GL_TEXTURE_2D, texName[TexCar] );
 glTexEnvf( GL_TEXTURE_2D, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 // if wrap is true, the texture wraps over at the edges (repeat)
 //       ... false, the texture ends at the edges (clamp)
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap ? GL_REPEAT : GL_CLAMP );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap ? GL_REPEAT : GL_CLAMP );
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_RGB, GL_UNSIGNED_BYTE, bmpFile->pixels);
 SDL_FreeSurface(bmpFile);

}

//
// CUBEMAP
void init_cube_map(){
		glPixelStorei(GL_UNPACK_ALIGNMENT, 4);

		/* definiamo il wrapping delle coordinate texture per ogni direzione*/
		glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_R, GL_REPEAT);

		/*definiamo come deve essere fatto il sampling nel caso di MAGNIFICATION e di MINIFICATION */
		glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

		SDL_Surface* bmpFile;

		bmpFile = SDL_LoadBMP(TexCubeXposFile);
		// Carica la texture per il piano ortogonale a X su x positivo
		gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
		SDL_FreeSurface(bmpFile);

		bmpFile = SDL_LoadBMP(TexCubeXnegFile);
		// Carica la texture per il piano ortogonale a X su x negativo
		gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
		SDL_FreeSurface(bmpFile);

		bmpFile = SDL_LoadBMP(TexCubeYposFile);
		// Carica la texture per il piano ortogonale a Y su y positivo
		gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
		SDL_FreeSurface(bmpFile);	

		bmpFile = SDL_LoadBMP(TexCubeYnegFile);
		// Carica la texture per il piano ortogonale a Y su y negativo
		gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
		SDL_FreeSurface(bmpFile);

		bmpFile = SDL_LoadBMP(TexCubeZposFile);
		// Carica la texture per il piano ortogonale a Z su z positivo
		gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
		SDL_FreeSurface(bmpFile);

		bmpFile = SDL_LoadBMP(TexCubeZnegFile);
		// Carica la texture per il piano ortogonale a Z su z negativo
		gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
		SDL_FreeSurface(bmpFile);

		/*stabilisci come devono essere generate le coordinate texture */
		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);
		glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);



}

void init_cube_map_tunnel(){
glPixelStorei(GL_UNPACK_ALIGNMENT, 4);

		/* definiamo il wrapping delle coordinate texture per ogni direzione*/
		glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_R, GL_REPEAT);

		/*definiamo come deve essere fatto il smapling nel caso di MAGNIFICATION e di MINIFICATION */
		glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

		SDL_Surface* bmpFile;

		bmpFile = SDL_LoadBMP(TexCarFile);
		// Carica la texture per il piano ortogonale a X su x positivo
		gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
		SDL_FreeSurface(bmpFile);

		bmpFile = SDL_LoadBMP(TexCarFile);
		// Carica la texture per il piano ortogonale a X su x negativo
		gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
		SDL_FreeSurface(bmpFile);

		bmpFile = SDL_LoadBMP(TexCarFile);
		// Carica la texture per il piano ortogonale a Y su y positivo
		gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
		SDL_FreeSurface(bmpFile);	

		bmpFile = SDL_LoadBMP(TexCarFile);
		// Carica la texture per il piano ortogonale a Y su y negativo
		gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
		SDL_FreeSurface(bmpFile);

		bmpFile = SDL_LoadBMP(TexCarFile);
		// Carica la texture per il piano ortogonale a Z su z positivo
		gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
		SDL_FreeSurface(bmpFile);

		bmpFile = SDL_LoadBMP(TexCarFile);
		// Carica la texture per il piano ortogonale a Z su z negativo
		gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
		SDL_FreeSurface(bmpFile);

		/*stabilisci come devono essere generate le coordinate texture */
		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);
		glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);



}
void enable_cube_map(){
	/*Abilita la generazione di coordinate texture */
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
	glEnable(GL_TEXTURE_GEN_R);	
	/* definisci come il risultato del texturing deve influire sul colore finale del frammento.*/
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_CLAMP);
	/*abilita il cube mapping */
    glEnable(GL_TEXTURE_CUBE_MAP_EXT);  
	//glEnable(GL_TEXTURE_CUBE_MAP);  
    glEnable(GL_NORMALIZE);	
}
void disable_cube_map(){
   glDisable(GL_TEXTURE_GEN_S);
   glDisable(GL_TEXTURE_GEN_T);
   glDisable(GL_TEXTURE_GEN_R);
   //glDisable(GL_TEXTURE_CUBE_MAP_EXT);	
   glDisable(GL_TEXTURE_CUBE_MAP);
}
//
//MACCHINA
void loadCarMesh(){
	// Generic loading of the mesh from disk
	if(vcg::tri::io::Importer<CMesh>::Open(mesh,CAR_MODEL_PATH)!=0) {
		fprintf(stderr,"Error reading file %s\n",CAR_MODEL_PATH);
		exit(0);
	}

	// Initialize the mesh itself
	vcg::tri::UpdateBounding<CMesh>::Box(mesh);      // update bounding box
	vcg::tri::UpdateNormals<CMesh>::PerVertexPerFace(mesh); // update Normals
		
	// Initialize the wrapper
	glWrap.m = &mesh;
	glWrap.Update();

}



//disegno la macchina
void draw_car_model(){


    
	//elimino le facce non in vista
	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK); 

	glPushAttrib(GL_CURRENT_BIT);
	glPushMatrix();

		float d=mesh.bbox.Diag();
		glTranslatef(0.0,0.5,0.0);
		glScale(3/d);
		glRotatef(-90,1,0,0);
		glRotatef(90,0,0,1);
		glTranslate(-glWrap.m->bbox.Center());	
		glColor3f(0.2,0.2,0.8);
		glWrap.Draw<vcg::GLW::DMSmooth,vcg::GLW::CMPerFace ,vcg::GLW::TMNone> (); //x vedere i colori macchina
		//glWrap.Draw<vcg::GLW::DMSmooth,   vcg::GLW::CMNone,vcg::GLW::TMNone> ();

	glPopMatrix();
	glPopAttrib();
	
	
		//VARIABILI PER LA POSIZIONE DEI FARI
		vcg::Point4f spotpos	=	vcg::Point4f ( -2.5 , 20 , -2.5 ,1.0);
		vcg::Point4f spotpos2	=	vcg::Point4f (  2.5 , 20, -2.50 ,1.0);
		vcg::Point4f spotdir	=	vcg::Point4f (0.0, -0.5, -2, 0.0);

		glLightfv(GL_LIGHT1,GL_POSITION,&spotpos[0]);
		glLightfv(GL_LIGHT1,GL_SPOT_DIRECTION, &spotdir[0] );
		glLightfv(GL_LIGHT2,GL_POSITION,&spotpos2[0]);
		glLightfv(GL_LIGHT2,GL_SPOT_DIRECTION, &spotdir[0] );


	
	glDisable(GL_CULL_FACE);

	
}

//
//LUCI
void init_lights(){
 glEnable(GL_LIGHTING);
 glEnable(GL_LIGHT0);
 glEnable(GL_LIGHT1);
 glEnable(GL_LIGHT2);

}
//definisci il materiale
void set_material_vect(GLfloat emissione[],GLfloat ambiente[],GLfloat diffusa[],GLfloat speculare[]){


	glMaterialfv(GL_FRONT,GL_EMISSION, &emissione[0]);
	glMaterialfv(GL_FRONT,GL_AMBIENT, &ambiente[0]);
	glMaterialfv(GL_FRONT,GL_DIFFUSE, &diffusa[0]);
	glMaterialfv(GL_FRONT,GL_SPECULAR, &speculare[0]);
	glMaterialf(GL_FRONT,GL_SHININESS,100); 

}
// setta le luci
void set_lights(){


	GLfloat emissione[] = { 0.0f, 0.0f, 0.0f, 1.0 };
	GLfloat ambiente[] = { 0.4f, 0.4f, 0.4f, 1.0 };
	GLfloat diffusa[] = { 0.8f, 0.8f, 0.8f, 1.0 };
	GLfloat speculare[] = { 1.0f, 1.0f, 1.0f, 1.0 };

	set_material_vect(emissione,ambiente,diffusa,speculare);
	
	/* setta i parametri della luce solare*/

	const GLfloat sun[] = { 0.2f, 0.2f, 0.2f, 1.0, 
							0.85f, 0.85f, 0.8f, 1.0,
							0.95f, 0.95f, 0.9f, 1.0};

	glLightfv(GL_LIGHT0,GL_AMBIENT,  &sun[0]);
	glLightfv(GL_LIGHT0,GL_DIFFUSE,  &sun[4]);
	glLightfv(GL_LIGHT0,GL_SPECULAR, &sun[8]);

	//vcg::Point4f sun_position = vcg::Point4f(0.0,5000.0,0.0,1.0);
	vcg::Point4f sun_position = vcg::Point4f(0.0,1000.0,-1000.0,1.0);
	glLightfv(GL_LIGHT0,GL_POSITION,&sun[0]);

	GLfloat modello_ambiente[]={ 0.2f, 0.2f, 0.2f, 1.0 };
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT,&modello_ambiente[0]);



	GLfloat luce_fari_ambiente[]={0.4,0.4,0.4,1.0};
	GLfloat luce_fari_diffuse[]={0.7,0.7,0.7,1.0};
	GLfloat luce_fari_specular[]={0.1,0.1,1.0,1.0};

	//GLfloat light_ambient2[]={1.0,0.0,0.0,1.0};


	
	//spotlight dx
	glLightfv(GL_LIGHT1,GL_AMBIENT,  &luce_fari_ambiente[0]);
	glLightfv(GL_LIGHT1,GL_DIFFUSE,  &luce_fari_diffuse[0]);
	glLightfv(GL_LIGHT1,GL_SPECULAR, &luce_fari_specular[0]);	
	glLighti(GL_LIGHT1,GL_SPOT_EXPONENT, 50);	 
	glLightf(GL_LIGHT1,GL_SPOT_CUTOFF, 30 );
	glLightf (GL_LIGHT1, GL_LINEAR_ATTENUATION,    0.0f);
	glLightf (GL_LIGHT1, GL_QUADRATIC_ATTENUATION, 0.0f);
	glLightf (GL_LIGHT1, GL_CONSTANT_ATTENUATION,  0.0f); 

	//spotlight sx
	glLightfv(GL_LIGHT2,GL_AMBIENT,  &luce_fari_ambiente[0]);
	glLightfv(GL_LIGHT2,GL_DIFFUSE,  &luce_fari_diffuse[0]);
	glLightfv(GL_LIGHT2,GL_SPECULAR, &luce_fari_specular[0]);	
	glLighti(GL_LIGHT2,GL_SPOT_EXPONENT, 50 );	 
	glLightf(GL_LIGHT2,GL_SPOT_CUTOFF, 30 );
	glLightf (GL_LIGHT2, GL_LINEAR_ATTENUATION,    0.0f);
	glLightf (GL_LIGHT2, GL_QUADRATIC_ATTENUATION, 0.0f);
	glLightf (GL_LIGHT2, GL_CONSTANT_ATTENUATION,  0.0f);
	// problema: se imposto diversamente i parametri di attenuazione non vedo piu' le luci dei fari

}

// da usare dentro i tunnel
void set_light_tunnel(bool in){
	if (in==true){
	const GLfloat sun[] = { 0.2f, 0.2f, 0.2f, 1.0, 
							0.3f, 0.3f, 0.3f, 1.0,
							0.95f, 0.95f, 0.9f, 1.0};

	glLightfv(GL_LIGHT0,GL_AMBIENT,  &sun[0]);
	glLightfv(GL_LIGHT0,GL_DIFFUSE,  &sun[4]);
	glLightfv(GL_LIGHT0,GL_SPECULAR, &sun[8]);
	}
	else{
	const GLfloat sun[] = { 0.2f, 0.2f, 0.2f, 1.0, 
							0.85f, 0.85f, 0.8f, 1.0,
							0.95f, 0.95f, 0.9f, 1.0};

	glLightfv(GL_LIGHT0,GL_AMBIENT,  &sun[0]);
	glLightfv(GL_LIGHT0,GL_DIFFUSE,  &sun[4]);
	glLightfv(GL_LIGHT0,GL_SPECULAR, &sun[8]);
	}
}
//
//OPENGL
/** inizializzazione lo stato di opengl */
void init_opengl(){
	
	// carica le textures
	initTexture();
	// carica il modello della macchina
	loadCarMesh();
	// inizializza le luci
	init_lights();
	
	// azzero delle variabili di appoggio per il punto di vista
	x=0;
	y=0;
	z=0;

	glShadeModel(GL_SMOOTH); //tipologia di sfumatura della luce
	glEnable(GL_NORMALIZE);
	glEnable(GL_DEPTH_TEST); // abilita il test di profondita'
	glDepthFunc(GL_LESS);
	glDepthRange(0,1);
	glClearDepth(1.0);


	glPolygonMode(GL_FRONT,GL_FILL); // rasterizza facce piene con la normale rivolta verso l'osservatore 
	glEnable(GL_COLOR_MATERIAL);     // abilita la possibilit di passare le caratteristiche del materiale con glColor



}

/** callback function for viewport resize.*/
void resize_screen(int w, int h){

	glClearAccum(0.0,0.0,0.0,1.0);
	glClear(GL_ACCUM_BUFFER_BIT);
	width_wps = w;
	height_wps = h;
	glViewport(0,0,width_wps,height_wps);
}


/** draw reference system*/
void draw_axis(){

	glPushAttrib(GL_CURRENT_BIT);
	glDisable(GL_LIGHTING);
	glBegin(GL_LINES);
	glColor3f(1.0,0.0,0.0);
	glVertex3f(0,0,0); //   X
	glVertex3f(10,0,0); 
  
	glColor3f(0.0,1.0,0.0);
    glVertex3f(0,0,0); //   Y
    glVertex3f(0,10,0);
  
	glColor3f(0.0,0.0,1.0);
    glVertex3f(0,0,0); //   Z
    glVertex3f(0,0,10);
    glEnd();
 	glPopAttrib();
}


//FARI
//proietto i fari
//non funziona!!!!
void draw_fari(void){

		glBindTexture( GL_TEXTURE_2D, texName[TexFari]);
		/* 1 */
		/// abilita la generazione automatica di coordinate texture.
		/// ora le coordinate texture sono generate dalla posizione del vertice
	 	glEnable( GL_TEXTURE_GEN_S );
		glEnable( GL_TEXTURE_GEN_T );
		glEnable( GL_TEXTURE_GEN_R );
		glEnable( GL_TEXTURE_GEN_Q );

		/// specificagl 
		vcg::Matrix44d proj; 

		glPushMatrix();
		glLoadIdentity();

		/* 2 */
		// le seguenti 2 trasformazioni mappano [-1,1]x[-1,1] in [0,1]x[0,1]
		glTranslatef(0.5,0.5,0.5);
		glScalef(0.5,0.5,0.5);

		/* 3 */
		// le seguenti trasformazioni definiscono la "vista" della texture proiettiva.
		// Se avete un fanale, ad esempio, queste trasformazioni costruicono la vista dal fanale
		// IMPORTANTISSIMO: queste trasformazioni sono intese (vedi sotto) nello spazio di vista!!
		// Quindi la  gluLookAt(0,0,0 equivale a dire che il centro di proiezione del proiettore
		// della texture  solidale con il punto di vista. 
		//glFrustum(-1,1,-1,1,5,10);
		gluLookAt(0,-4,-4, 0,-0.01,-10 ,0,1,0);

		//VARIABILI PER LA POSIZIONE DEI FARI

		vcg::Point4f _spotpos	=	vcg::Point4f ( -4.5 , 3.5 , -15.5 ,1.0);
		vcg::Point4f _spotpos2	=	vcg::Point4f (  4.5 , 3.5 , -15.5 ,1.0);
		vcg::Point4f _spotdir	=	vcg::Point4f (0.0, -0.2, -1.0, 0.0);

		gluLookAt(_spotpos[0],_spotpos[1],_spotpos[2], _spotdir[0]+_spotpos[0],_spotdir[1]+_spotpos[1],_spotdir[2]+_spotpos[2],0,1,0);

		/* 4 */
		// rileggo la matrice da opengl
			glGetDoublev(GL_MODELVIEW_MATRIX,&proj[0][0]);

		// per passarla a glTexGen mi servira' trasposta
		proj = vcg::Transpose(proj);

		glPopMatrix();

		/* 5 */
		// specifica che la trasformazioni che verranno impostate per calcolare le cordinate texture
		// sono espresse nello spazio di vista (GL_EYE_LINEAR) e NON nello spazio oggetto (GL_OBJECT_LINEAR)
		glTexGeni( GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR );
		glTexGeni( GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR );
		glTexGeni( GL_R, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR );
		glTexGeni( GL_Q, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR );


		/* 6 */
		// (finalmente) specifica le trasformazioni
		// Notare che la trasformazione viene definita al momento della chiamata a glTexGen e che i piani passati,
		// ovvero la matrice proj, vengono  trasformati per l'inversa della matrice MODEL_VIEW, che noi
		// abbiamo appositamente posto all'identita'
		// Il risultato finale  che le posizioni dei vertici subiscono le trasformazioni specificate in 2 e 3
		glTexGendv( GL_S, GL_EYE_PLANE, &proj[0][0] );
		glTexGendv( GL_T, GL_EYE_PLANE, &proj[1][0] );
		glTexGendv( GL_R, GL_EYE_PLANE, &proj[2][0] );
		glTexGendv( GL_Q, GL_EYE_PLANE, &proj[3][0] );

		glDisable( GL_TEXTURE_GEN_S );
		glDisable( GL_TEXTURE_GEN_T );
		glDisable( GL_TEXTURE_GEN_R );
		glDisable( GL_TEXTURE_GEN_Q );

}

//
//FORME
// disegna un cubo unitario centrato sull origine, 
void draw_unit_cube(){

	// faccia 0,0,1 
  glBegin(GL_QUADS);
    glVertex3d( -1.0, -1.0, 1.0);
    glVertex3d(  1.0, -1.0, 1.0);
    glVertex3d(  1.0,  1.0, 1.0);
    glVertex3d( -1.0,  1.0, 1.0);
  glEnd();

  // faccia 1,0,0 
  glBegin(GL_QUADS);
    glVertex3d( 1.0, -1.0,  1.0);
    glVertex3d( 1.0, -1.0, -1.0);
    glVertex3d( 1.0,  1.0, -1.0);
    glVertex3d( 1.0,  1.0,  1.0);
  glEnd();
  
  // faccia 1,0,0 
  glBegin(GL_QUADS);
    glVertex3d( 1.0, -1.0,  1.0);
    glVertex3d( 1.0, -1.0, -1.0);
    glVertex3d( 1.0,  1.0, -1.0);
    glVertex3d( 1.0,  1.0,  1.0);
  glEnd();

  // faccia 0,0,-1 
  glBegin(GL_QUADS);
    glVertex3d( 1.0, -1.0, -1.0);
    glVertex3d(-1.0, -1.0, -1.0);
    glVertex3d(-1.0,  1.0, -1.0);
    glVertex3d( 1.0,  1.0, -1.0);
  glEnd();

  // faccia -1,0,0 
  glBegin(GL_QUADS);
    glVertex3d(-1.0, -1.0, -1.0);
    glVertex3d(-1.0, -1.0,  1.0);
    glVertex3d(-1.0,  1.0,  1.0);
    glVertex3d(-1.0,  1.0, -1.0);
  glEnd();
  
  // faccia  0,1,0 
  glBegin(GL_QUADS);
    glVertex3d(-1.0,  1.0,  1.0);
    glVertex3d( 1.0,  1.0,  1.0);
    glVertex3d( 1.0,  1.0, -1.0);
    glVertex3d(-1.0,  1.0, -1.0);
  glEnd();

  // faccia  0,-1,0 
  glBegin(GL_QUADS);
    glVertex3d(-1.0, -1.0, -1.0);
    glVertex3d( 1.0, -1.0, -1.0);
    glVertex3d( 1.0, -1.0,  1.0);
    glVertex3d(-1.0, -1.0,  1.0);
  glEnd();
}

/* disegna un cilindro di raggio 1 e alto 1   allineato con l'asse Y e centrato in 0,0,0 */
void draw_cylinder(){

	int slices = 20;
	float step = 2*M_PI/slices;

	for(int i = 0; i < slices;++i){

		glBegin(GL_QUADS);
		glVertex3f( cos(i*step),-0.5,-sin(i*step));
		glVertex3f( cos((i+1)*step),-0.5,-sin((i+1)*step));
		glVertex3f( cos((i+1)*step), 0.5,-sin((i+1)*step));
		glVertex3f( cos(i*step), 0.5,-sin(i*step));
		glEnd();

		glBegin(GL_TRIANGLES);
		glVertex3f( cos(i*step),0.5,-sin(i*step) );
		glVertex3f( cos((i+1)*step),0.5,-sin((i+1)*step)  );
		glVertex3f( 0.0,0.5, 0.0);
		glVertex3f( cos(i*step),-0.5,-sin(i*step) );
		glVertex3f( 0.0,-0.5,0.0 );
		glVertex3f( cos((i+1)*step),-0.5,-sin((i+1)*step)  );
		glEnd();
	}

}

void draw_cylinder(int slices){

	//int slices = 20;
	float step = 2*M_PI/slices;

	for(int i = 0; i < slices;++i){
		glBegin(GL_QUADS);
		glVertex3f( cos(i*step),-0.5,-sin(i*step));
		glVertex3f( cos((i+1)*step),-0.5,-sin((i+1)*step));
		glVertex3f( cos((i+1)*step), 0.5,-sin((i+1)*step));
		glVertex3f( cos(i*step), 0.5,-sin(i*step));
		glEnd();

		glBegin(GL_TRIANGLES);
		glVertex3f( cos(i*step),0.5,-sin(i*step) );
		glVertex3f( cos((i+1)*step),0.5,-sin((i+1)*step)  );
		glVertex3f( 0.0,0.5, 0.0);
		glVertex3f( cos(i*step),-0.5,-sin(i*step) );
		glVertex3f( 0.0,-0.5,0.0 );
		glVertex3f( cos((i+1)*step),-0.5,-sin((i+1)*step)  );
		glEnd();
	}

}

//scrivi su schermo
void gl_print(float x, float y, const char *str) {
	
glDisable(GL_TEXTURE_2D);
glDisable(GL_LIGHTING);

  glRasterPos2f(x, y);
  int len = (int)strlen(str);
  for(int i = 0; i < len; i++) 
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, str[i]);

glEnable(GL_LIGHTING);
glEnable(GL_TEXTURE_2D);
}




//
//TUNNEL
// disegno i tunnel
void draw_tunnel(void){ 

	glEnable(GL_TEXTURE_2D);
	glBindTexture( GL_TEXTURE_2D, texName[Textunnel]);

	Street::TunnelsIterator tunit; 
	float slices = 30;
	float step = M_PI/slices;
        // per ogni tunnel 
        for(tunit=race.circuit->street.tunnels.begin();tunit!=race.circuit->street.tunnels.end();tunit++){ 


                std::vector<vcg::Point2<Vertex>> base=tunit->path; 
                std::vector<vcg::Point2<Vertex>>::iterator bt; 
                // accedi alla sua  altezza 
                float heigth=tunit->heigth; 


                // accedi alla sua base 
                for(bt=base.begin();bt!=base.end()-1;bt++){ 
					
					vcg::Point3f p0,p1,p01,p11;
					vcg::Point3f p_cur_b,p_next_b,p_cur_f,p_next_f;
					float y_cur,y_next,alpha;

					p0 = (*bt)[0].P(); //p0 sono i vertici dx del tunnel
					p1 = (*bt)[1].P(); //p1 sono i vertici sx del tunnel
					++bt;
					p01= (*bt)[0].P();//dx sopra
					p11= (*bt)[1].P();//sx sopra
					--bt;

					glPushMatrix(); 
					glPushAttrib(GL_CURRENT_BIT);
						p_cur_b=p0;
						p_cur_f=p01;
						y_cur=0.0;
							
					glBegin(GL_QUAD_STRIP);
						for(int i = 0; i < slices+1;i++){
							
							alpha=i/slices;
							p_next_b=p0*(1-alpha)+p1*(alpha);
							p_next_f=p01*(1-alpha)+p11*(alpha);
							y_next=heigth*sin((i+1)*step);///2.0f;
						
						 
							//glColor3f(0.6,0.5,0.5);
							glTexCoord2f(alpha, 1.0);					
							glVertex3f(p_cur_b[0],y_cur,p_cur_b[2]); 
							glTexCoord2f(alpha,0.0);							
							glVertex3f(p_cur_f[0],y_cur,p_cur_f[2]);
						


							p_cur_b=p_next_b;
							p_cur_f=p_next_f;
							y_cur=y_next;

						}
					glEnd();

					glPopAttrib();
					glPopMatrix();

			

                } 


        } 

}
//equazione implicita della retta(utilizzata controllare se l'auto e' nel tunnel o meno)
bool EqImpRetta(vcg::Point3f punto_uno,vcg::Point3f punto_due,vcg::Point3f car_position){
	float dx, dz;
	float risultato;
	bool min_mag=false; //variabile che diviene vera quando X/dx + Z/dz > 1

	
		dx=punto_due.X()-punto_uno.X();
		dz=punto_due.Z()-punto_uno.Z();

		risultato=(car_position.Z()-punto_uno.Z())*dx-(car_position.X()-punto_uno.X())*dz;


	if (risultato>=0) min_mag=true;

	return min_mag;
	
	
	
}
//la distanza tra un punto e l'altro e' magiore  di un tot?
bool distanza_min(vcg::Point3f punto_uno,vcg::Point3f punto_due,float tot){

float distanza=0;
float membro1=(punto_uno.X()-punto_due.X());
float membro2=(punto_uno.Z()-punto_due.Z());
distanza=sqrt( membro1*membro1 + membro2*membro2 );


bool ret=false; 
if (distanza<=tot) ret=true;
return ret;
}

// sono in un tunnel?
bool IsInT(vcg::Point3f car_position){
	bool IsIn=false; // variabile che diviene true quando sono in un tunel
	Street::TunnelsIterator tunit; 

        // per ogni tunnel 
        for(tunit=race.circuit->street.tunnels.begin();tunit!=race.circuit->street.tunnels.end();tunit++){ 

                std::vector<vcg::Point2<Vertex>> base=tunit->path; 
                std::vector<vcg::Point2<Vertex>>::iterator bt; 
                // accedi alla sua  altezza 
                float heigth=tunit->heigth; 
                // accedi alla sua base 
				for(bt=base.begin();bt!=base.end()-1;bt++){

					vcg::Point3f p0,p1,p01,p11;
					p0 = (*bt)[0].P(); //p0 sono i vertici dx del tunnel
					p1 = (*bt)[1].P(); //p1 sono i vertici sx del tunnel
					++bt;
					p01= (*bt)[0].P();//dx sopra
					p11= (*bt)[1].P();//sx sopra
					--bt;
					bool distanza=(distanza_min(p0,car_position,10.0f) | distanza_min(p1,car_position,10.0f) | distanza_min(p01,car_position,10.0f) | distanza_min(p11,car_position,10.0f));
					if(distanza==true){
						bool InVert=false;
						bool InOriz=false;					
						bool temp1=EqImpRetta(p0,p1,car_position);
						bool temp2=EqImpRetta(p01,p11,car_position);
						InOriz=(temp1)^(temp2);
						if (InOriz==true){
							bool temp3=EqImpRetta(p0,p01,car_position);
							bool temp4=EqImpRetta(p1,p11,car_position);					
							InVert=(temp3)^(temp4);
						}	
						if(InOriz & InVert)IsIn=true;
						
					}
				}
				}

		return IsIn;
}
//
//PALAZZI
void draw_palazzi(){

	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK); 
	glEnable(GL_TEXTURE_2D);
	glBindTexture( GL_TEXTURE_2D, texName[TexBuilding]);

	Circuit::BuildingsIterator bi; //iteratore sui palazzi

	float height=0.0; //altezza palazzo
	for(bi=race.circuit->buildings.begin();bi!=race.circuit->buildings.end();++bi){

		height=(*bi).height*30;

		std::vector<Vertex>::iterator vi; //iteratore sui vertici di ciascun palazzo

		for(vi=(*bi).footprint.begin(); vi!=(*bi).footprint.end()-1;++vi){

			glBegin(GL_QUADS);

				vcg::Point3f p00 = (*vi).P();
				++vi;
				vcg::Point3f p01 = (*vi).P();
				--vi;

				glTexCoord2f(0.0,0.0);glVertex3f(p00[0],p00[1]+height,p00[2]);
				glTexCoord2f(1.0,0.0);glVertex3f(p01[0],p01[1]+height,p01[2]);
				glTexCoord2f(1.0,1.0);glVertex3f(p01[0],p01[1],p01[2]);
				glTexCoord2f(0.0,1.0);glVertex3f(p00.X(),p00.Y(),p00.Z());


	
			glEnd();
		}

		//ultimo lato
		glBegin(GL_QUADS);

			vi=(*bi).footprint.begin();
			vcg::Point3f p01=(*vi).P();	
			vi=(*bi).footprint.end()-1;
			vcg::Point3f p00=(*vi).P();

			glTexCoord2f(0.0,0.0);glVertex3f(p00[0],p00[1]+height,p00[2]);
			glTexCoord2f(1.0,0.0);glVertex3f(p01[0],p01[1]+height,p01[2]);
			glTexCoord2f(1.0,1.0);glVertex3f(p01[0],p01[1],p01[2]);
			glTexCoord2f(0.0,1.0);glVertex3f(p00[0],p00[1],p00[2]);

		glEnd();


		//tetto
		glBegin(GL_QUADS);

			vi=(*bi).footprint.begin();
			p00=(*vi).P();	
			++vi;
			vcg::Point3f p10=(*vi).P();
			++vi;
			vcg::Point3f p11=(*vi).P();
			++vi;
			p01=(*vi).P();

			/*glTexCoord2f(0.0,1.0);*/glVertex3f(p01[0],p01[1]+height,p01[2]);
			/*glTexCoord2f(1.0,1.0);*/glVertex3f(p11[0],p11[1]+height,p11[2]);
			/*glTexCoord2f(1.0,0.0);*/glVertex3f(p10[0],p10[1]+height,p10[2]);
			/*glTexCoord2f(0.0,0.0);*/glVertex3f(p00[0],p00[1]+height,p00[2]);

		glEnd();



	}

	



	glDisable(GL_TEXTURE_2D);
	glDisable(GL_CULL_FACE);
	}

//
//ALBERI
//billboarding procedure
void billboard(float *cam, float *worldPos) {
	
	float lookAt[3]={0,0,1};//il quad guarda verzo +z
	float objToCamProj[3];//vettore dalla camera all'albero, proiettato su XZ
	float objToCam[3]; //vettore dalla camera all'albero
	float upAux[3]; //prodotto vettoriale tra objToCamProj e lookAt --> mi d l'orientazione dell'angolo
	float coseno;

//vettore individuato dalla posizone del punto di vista e dalla posizione dell'albero
//proiettato sul piano XZ
	objToCamProj[0] = cam[0] - worldPos[0] ;
	objToCamProj[1] = 0;
	objToCamProj[2] = cam[2] - worldPos[2] ;

// normalizzazione del vettore per calcolarci poi il coseno
	float d = (sqrt((objToCamProj[0]*objToCamProj[0]) + (objToCamProj[1]*objToCamProj[1]) + (objToCamProj[2]*objToCamProj[2])));
	objToCamProj[0] = objToCamProj[0] / d;
	objToCamProj[1] = objToCamProj[1] / d;
	objToCamProj[2] = objToCamProj[2] / d;

// determino quando l'angolo  positivo o negativo (prodotto vettoriale)
// se l'angolo  positivo upAux  un vettore che punta nella direzione positiva delle y
// altrimenti punter in basso e la rotazione sar invertita
	(upAux)[0] = (lookAt)[1] * (objToCamProj)[2] - (objToCamProj)[1] * (lookAt)[2]; 
	(upAux)[1] = (lookAt)[2] * (objToCamProj)[0] - (objToCamProj)[2] * (lookAt)[0]; 
	(upAux)[2] = (lookAt)[0] * (objToCamProj)[1] - (objToCamProj)[0] * (lookAt)[1];

// calcolo il prodotto scalare tra i 2 vettori == cos(angolo)
coseno = ((lookAt)[0] * (objToCamProj)[0] + (lookAt)[1] * (objToCamProj)[1] + (lookAt)[2] * (objToCamProj)[2]);

// rotazione 
// The if statement is used for stability reasons
// if the lookAt and v vectors are too close together then |aux| could
// be bigger than 1 due to lack of precision
	if ((coseno < 0.99990) && (coseno > -0.9999))
		glRotatef(acos(coseno)*180/3.14,upAux[0], upAux[1], upAux[2]);	


// The second part tilts the object so that it faces the camera

//vettore individuato dalla posizone del punto di vista e la posizione dell'alberello
	objToCam[0] = cam[0] - worldPos[0] ;
	objToCam[1] = cam[1] - worldPos[1] ;
	objToCam[2] = cam[2] - worldPos[2] ;

// normalizzazione del vettore
	d = (sqrt((objToCam[0]*objToCam[0]) + (objToCam[1]*objToCam[1]) + (objToCam[2]*objToCam[2])));
	objToCam[0] = objToCam[0] / d;
	objToCam[1] = objToCam[1] / d;
	objToCam[2] = objToCam[2] / d;

// prodotto scalare x trovare il coseno dell'angolo tra objToCamProj e objToCam
// mi serve x stabilire la rotazione rispetto all'asse X --> inclinazione
	coseno = ((objToCamProj)[0] * (objToCam)[0] + (objToCamProj)[1] * (objToCam)[1] + (objToCamProj)[2] * (objToCam)[2]);


// rotazione attorno all'asse x per migliorare la vista dall'alto
// The test is done to prevent instability when objToCam and objToCamProj have a very small
// angle between them
	if ((coseno < 0.99990) && (coseno> -0.9999))
		if (objToCam[1] < 0)
			glRotatef(acos(coseno)*180/3.14,1,0,0);	
		else
			glRotatef(acos(coseno)*180/3.14,-1,0,0);	

}




/** draw trees (billboard)**/
void draw_billboard_trees(){

	glEnable(GL_TEXTURE_2D);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_ALPHA_TEST);
	glAlphaFunc(GL_GREATER, 0);
	glBindTexture(GL_TEXTURE_2D, texName[TexTree]);

	Circuit::TreesIterator ti;
	//std::vector<Circuit::Tree>::iterator ti; //iteratore alberi 
	for(ti=race.circuit->trees.begin();ti!=race.circuit->trees.end();++ti){
		glPushMatrix();
			vcg::Point3f pos=(*ti).pos;
			float height = (*ti).height*5;
			float cam[3]={pov[0]+x,pov[1]+y,pov[2]+x};
			float worldPos[3]={pos[0],pos[1],pos[2]};
			glTranslatef(worldPos[0],worldPos[1],worldPos[2]);

			billboard(cam,worldPos);

			glBegin(GL_QUADS);
				glTexCoord2f(0,0);glVertex3f(-(height), 0.0f, 0.0f);
				glTexCoord2f(1,0);glVertex3f((height), 0.0f, 0.0f);
				glTexCoord2f(1,1);glVertex3f((height), height*2,  0.0f);
				glTexCoord2f(0,1);glVertex3f(-(height), height*2,  0.0f);
			glEnd();

		glPopMatrix();
	}
	glDisable(GL_BLEND);
    glDisable(GL_ALPHA_TEST);
	glDisable(GL_TEXTURE_2D);
	
}



//
//TERRENO
//disegna il terreno con un solo poligono
void draw_terrain(vcg::Point3f bmin,vcg::Point3f bmax){
		// draw bounding box of the terrain & apply green texture
	glPushMatrix();
	glPushAttrib(GL_CURRENT_BIT);

	glNormal3f(0.0,1.0,0.0);

	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	//glColor3f(0.2,0.9,0.3); //colore verde sotto la texture dell'erba
	glBindTexture( GL_TEXTURE_2D, texName[TexGrass]);

	glBegin(GL_QUADS);	
	glTexCoord2f(0.0,0.0);
	glVertex3f(bmin[0],bmin[1],bmin[2]);
	glTexCoord2f(1.0,0.0);
	glVertex3f(bmax[0],bmin[1],bmin[2]);
	glTexCoord2f(0.0,1.0);
	glVertex3f(bmax[0],bmin[1],bmax[2]);
	glTexCoord2f(1.0,1.0);
	glVertex3f(bmin[0],bmin[1],bmax[2]);
	glEnd();

	glPopAttrib();
	glPopMatrix();
	
	
}

//disegna il tereno con triangolazione
void draw_terrain2(const int &n,vcg::Point3f bmin,vcg::Point3f bmax){


	


	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK); 
	glPushMatrix();
	glPushAttrib(GL_CURRENT_BIT);
	
	//glTranslatef(pov.X(),0,pov.Z());

	float lato_x	=	bmax[0]-bmin[0];
	float passo_x	=	lato_x/float(n);

	float lato_z	=	bmax[2]-bmin[2];
	float passo_z	=	lato_z/float(n);



	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	glColor3f(0.2,0.9,0.3); //colore verde sotto la texture dell'erba
	glBindTexture( GL_TEXTURE_2D, texName[TexGrass]);

	
	for (int x=0;x<n;x++){

		glBegin(GL_QUAD_STRIP);

		glNormal3f(0.0,1.0,0.0);

		for (int z=0;z<=n;z++){

			
			float x_tex=(float(x)/n);
			float x_tex_1=(float(x+1)/n);
			float z_tex=(float(z)/n);


			float index=z%2;
			glTexCoord2f(index,1);
			//glTexCoord2f(x_tex_1,z_tex);
			glVertex3f(bmin[0]+(x+1)*passo_x,0.0,bmin[2]+z*passo_z);	

			glTexCoord2f(index,0);
			//glTexCoord2f(x_tex,z_tex);
			glVertex3f(bmin[0]+x*passo_x,0.0,bmin[2]+z*passo_z);


		}
		glEnd();

	}



	glPopAttrib();
	glPopMatrix();
	glDisable(GL_CULL_FACE);


}



// applico la texture alla strada
void RoadTexture(vcg::Point3f p00, vcg::Point3f p10, vcg::Point3f p01, vcg::Point3f p11 ){

	float x00x,x10x,x01x,x11x;
	float x00y,x10y,x01y,x11y;
	float x00z,x10z,x01z,x11z;

	x00x=p00.X();
	x00y=p00.Y();
	x00z=p00.Z();

	x10x=p10.X();
	x10y=p10.Y();
	x10z=p10.Z();

	x01x=p01.X();
	x01y=p01.Y();
	x01z=p01.Z();

	x11x=p11.X();
	x11y=p11.Y();
	x11z=p11.Z();

	glTexCoord2f(0.0, 0.0); glVertex3f(x00x, x00y+0.1, x00z);//basso dx
	glTexCoord2f(1.0, 0.0); glVertex3f(x10x, x10y+0.1, x10z);//basso sx
	glTexCoord2f(0.0, 1.0); glVertex3f(x01x, x01y+0.1, x01z);//alto dx
	glTexCoord2f(1.0, 1.0); glVertex3f(x11x, x11y+0.1, x11z);//alto sx
}


// disegno la strada
void draw_road(void){
	// DISEGNO IL CIRCUITO
	
	glPushAttrib(GL_CURRENT_BIT);

	Street::TilesIterator ti;
	
	int i;
	glColor3f(0.5,0.5,0.5); //colore x la strada
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);// specifica la qualit del colore e dell'interpolazione delle texture
	glBindTexture( GL_TEXTURE_2D, texName[TexRoad]); // texture da usare

	glBegin(GL_QUAD_STRIP);
	glNormal3f(0,1,0);// definisco le normali tutte verso l'alto
	vcg::Point3f first0,first1;
	for(i=0,ti = race.circuit->street.path.begin(); ti != race.circuit->street.path.end();++ti,++i){
		vcg::Point3f p0,p1;
	
		p0 = (*ti)[0].P(); //p0 sono i vertici dx del circuito
		p1 = (*ti)[1].P(); //p1 sono i vertici sx del circuito
		if(i==0){first0=p0;first1=p1;}
		
		
		vcg::Point3f p01,p11;
		if(ti!=race.circuit->street.path.end()-1){
			++ti;
			p01= (*ti)[0].P();//dx sopra
			p11= (*ti)[1].P();//sx sopra
			RoadTexture(p0,p1,p01,p11);
			--ti;
		}
		else{
			RoadTexture(p0,p1,first0,first1);
		}
	}

	glEnd();

	glPopAttrib();
}

//
//SKYBOX
// disegna lo skybox alle estremit della mappa
void draw_skybox(vcg::Point3f bmin,vcg::Point3f bmax){


	glPushAttrib(GL_CURRENT_BIT);

	glPushMatrix();

	//glTranslatef(pov.X(),0/*pov.Y()*/,pov.Z());
	// draw the skybox
	// lato z positivi
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	//glColor3f(0.8,0.8,1.0);
	glBindTexture( GL_TEXTURE_2D, texName[TexSkyBoxZpos]);
	glBegin(GL_QUADS);	
	glTexCoord2f(0.999,0.999);
	glVertex3f(bmin[0],bmin[1],bmax[2]);
	glTexCoord2f(0.001,0.999);
	glVertex3f(bmax[0],bmin[1],bmax[2]);
	glTexCoord2f(0.001,0.001);
	glVertex3f(bmax[0],bmin[1]+3000,bmax[2]);
	glTexCoord2f(0.999,0.001);
	glVertex3f(bmin[0],bmin[1]+3000,bmax[2]);
	glEnd();
	// lato zeta negativi
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	//glColor3f(0.8,0.8,1.0);
	glBindTexture( GL_TEXTURE_2D, texName[TexSkyBoxZneg]);	
	glBegin(GL_QUADS);	
	glTexCoord2f(0.001,0.999);
	glVertex3f(bmin[0],bmin[1],bmin[2]);
	glTexCoord2f(0.999,0.999);
	glVertex3f(bmax[0],bmin[1],bmin[2]);
	glTexCoord2f(0.999,0.001);
	glVertex3f(bmax[0],bmin[1]+3000,bmin[2]);
	glTexCoord2f(0.001,0.001);
	glVertex3f(bmin[0],bmin[1]+3000,bmin[2]);
	glEnd();
	////lato y negativi

	//lato y positivi
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	//glColor3f(0.8,0.8,1.0);
	glBindTexture( GL_TEXTURE_2D, texName[TexSkyBoxYpos]);
	glBegin(GL_QUADS);	
	glTexCoord2f(0.001,0.001);
	glVertex3f(bmax[0],bmin[1]+3000,bmin[2]);
	glTexCoord2f(0.001,0.999);
	glVertex3f(bmax[0],bmin[1]+3000,bmax[2]);
	glTexCoord2f(0.999,0.999);
	glVertex3f(bmin[0],bmin[1]+3000,bmax[2]);
	glTexCoord2f(0.999,0.001);
	glVertex3f(bmin[0],bmin[1]+3000,bmin[2]);
	glEnd();
	
	

	//lato x positivi
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	//glColor3f(0.8,0.8,1.0);
	glBindTexture( GL_TEXTURE_2D, texName[TexSkyBoxXneg]);
	glBegin(GL_QUADS);	
	glTexCoord2f(0.001,0.999);
	glVertex3f(bmax[0],bmin[1],bmin[2]);
	glTexCoord2f(0.999,0.999);
	glVertex3f(bmax[0],bmin[1],bmax[2]);
	glTexCoord2f(0.999,0.001);
	glVertex3f(bmax[0],bmin[1]+3000,bmax[2]);
	glTexCoord2f(0.001,0.001);
	glVertex3f(bmax[0],bmin[1]+3000,bmin[2]);
	glEnd();
	//lato x negativi
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	//glColor3f(0.8,0.8,1.0);
	glBindTexture( GL_TEXTURE_2D, texName[TexSkyBoxXpos]);
	glBegin(GL_QUADS);	
	glTexCoord2f(0.999,0.999);
	glVertex3f(bmin[0],bmin[1],bmin[2]);
	glTexCoord2f(0.001,0.999);
	glVertex3f(bmin[0],bmin[1],bmax[2]);
	glTexCoord2f(0.001,0.001);
	glVertex3f(bmin[0],bmin[1]+3000,bmax[2]);
	glTexCoord2f(0.999,0.001);
	glVertex3f(bmin[0],bmin[1]+3000,bmin[2]);
	glEnd();

	glPopMatrix();
	glPopAttrib();
}



// disegna lo skybox attorno alla telecamera
void draw_skybox2(){

	 glDepthMask(false);

	float coord=0.5f;
	
	glPushMatrix();


 	// centro il cubo nel pov
	glTranslatef(pov.X(),pov.Y()+0.2f,pov.Z());
	glScalef(1.0,0.5,1.0); 


	glPushAttrib(GL_CURRENT_BIT);


	
	// draw the skybox
	// lato z positivi
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	//glColor3f(0.8,0.8,1.0);
	glBindTexture( GL_TEXTURE_2D, texName[TexSkyBoxZpos]);
	glBegin(GL_QUADS);	
	glTexCoord2f(0.999,0.999);
	glVertex3f(-coord,-coord,coord);
	glTexCoord2f(0.001,0.999);
	glVertex3f(coord,-coord,coord);
	glTexCoord2f(0.001,0.001);
	glVertex3f(coord,coord,coord);
	glTexCoord2f(0.999,0.001);
	glVertex3f(-coord,coord,coord);
	glEnd();
	// lato zeta negativi
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	//glColor3f(0.8,0.8,1.0);
	glBindTexture( GL_TEXTURE_2D, texName[TexSkyBoxZneg]);	
	glBegin(GL_QUADS);	
	glTexCoord2f(0.001,0.999);
	glVertex3f(-coord,-coord,-coord);
	glTexCoord2f(0.999,0.999);
	glVertex3f(coord,-coord,-coord);
	glTexCoord2f(0.999,0.001);
	glVertex3f(coord,coord,-coord);
	glTexCoord2f(0.001,0.001);
	glVertex3f(-coord,coord,-coord);
	glEnd();
	////lato y negativi

	//lato y positivi
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	//glColor3f(0.8,0.8,1.0);
	glBindTexture( GL_TEXTURE_2D, texName[TexSkyBoxYpos]);
	glBegin(GL_QUADS);	
	glTexCoord2f(0.001,0.001);
	glVertex3f(coord,coord,-coord);
	glTexCoord2f(0.001,0.999);
	glVertex3f(coord,coord,coord);
	glTexCoord2f(0.999,0.999);
	glVertex3f(-coord,coord,coord);
	glTexCoord2f(0.999,0.001);
	glVertex3f(-coord,coord,-coord);
	glEnd();
	
	

	//lato x positivi
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	//glColor3f(0.8,0.8,1.0);
	glBindTexture( GL_TEXTURE_2D, texName[TexSkyBoxXneg]);
	glBegin(GL_QUADS);	
	glTexCoord2f(0.001,0.999);
	glVertex3f(coord,-coord,-coord);
	glTexCoord2f(0.999,0.999);
	glVertex3f(coord,-coord,coord);
	glTexCoord2f(0.999,0.001);
	glVertex3f(coord,coord,coord);
	glTexCoord2f(0.001,0.001);
	glVertex3f(coord,coord,-coord);
	glEnd();


	//lato x negativi
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	//glColor3f(0.8,0.8,1.0);
	glBindTexture( GL_TEXTURE_2D, texName[TexSkyBoxXpos]);
	glBegin(GL_QUADS);	
	glTexCoord2f(0.999,0.999);
	glVertex3f(-coord,-coord,-coord);
	glTexCoord2f(0.001,0.999);
	glVertex3f(-coord,-coord,+coord);
	glTexCoord2f(0.001,0.001);
	glVertex3f(-coord,coord,coord);
	glTexCoord2f(0.999,0.001);
	glVertex3f(-coord,coord,-coord);
	glEnd();

	glPopMatrix();
	glPopAttrib();
	glDepthMask(true);

}



//cubemaps dinamico
void dinamic_cube_maps(void){
	vcg::Point3f bmin = race.circuit->bbox.min;
	vcg::Point3f bmax = race.circuit->bbox.max;

static bool s_inizialized=false;

glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
glViewport(0,0,width_wps,height_wps);
glMatrixMode(GL_PROJECTION);
glLoadIdentity();
gluPerspective(90,1,0.1,10000);
glMatrixMode(GL_MODELVIEW);



	// x positivi
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	gluLookAt(0,0,0,1,0,0,0,1,0);
	
	glEnable(GL_TEXTURE_2D);
	draw_skybox(bmin,bmax);
	draw_palazzi();
	glDisable(GL_TEXTURE_2D);
	
	glBindTexture(GL_TEXTURE_CUBE_MAP,texName[TexSkyBoxXposDin]);

	if (s_inizialized){
		glCopyTexSubImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X,0,0,0,0,0,width_wps,height_wps);
	}
	else{
		glCopyTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X,0,GL_RGB,0,0,width_wps,height_wps,0);
	}


	// x negativi
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	gluLookAt(0,0,0,-1,0,0,0,1,0);

	glEnable(GL_TEXTURE_2D);
	draw_skybox(bmin,bmax);
	draw_palazzi();
	glDisable(GL_TEXTURE_2D);

	glBindTexture(GL_TEXTURE_CUBE_MAP,texName[TexSkyBoxXnegDin]);

	if (s_inizialized){
		glCopyTexSubImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_X,0,0,0,0,0,width_wps,height_wps);
	}
	else{
		glCopyTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_X,0,GL_RGB,0,0,width_wps,height_wps,0);
	}

	// z positivi
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	gluLookAt(0,0,0,0,0,1,0,1,0);

	glEnable(GL_TEXTURE_2D);
	draw_skybox(bmin,bmax);
	draw_palazzi();
	glDisable(GL_TEXTURE_2D);

	glBindTexture(GL_TEXTURE_CUBE_MAP,texName[TexSkyBoxZposDin]);

	if (s_inizialized){
		glCopyTexSubImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_Z,0,0,0,0,0,width_wps,height_wps);
	}
	else{
		glCopyTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_Z,0,GL_RGB,0,0,width_wps,height_wps,0);
	}
	// z negativi
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	gluLookAt(0,0,0,0,0,-1,0,1,0);

	glEnable(GL_TEXTURE_2D);
	draw_skybox(bmin,bmax);
	draw_palazzi();
	glDisable(GL_TEXTURE_2D);

	glBindTexture(GL_TEXTURE_CUBE_MAP,texName[TexSkyBoxZnegDin]);

	if (s_inizialized){
		glCopyTexSubImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z,0,0,0,0,0,width_wps,height_wps);
	}
	else{
		glCopyTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z,0,GL_RGB,0,0,width_wps,height_wps,0);
	}

	// y positivi
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	gluLookAt(0,0,0,0,1,0,-1,0,0);

	glEnable(GL_TEXTURE_2D);
	draw_skybox(bmin,bmax);
	draw_palazzi();
	glDisable(GL_TEXTURE_2D);

	glBindTexture(GL_TEXTURE_CUBE_MAP,texName[TexSkyBoxYposDin]);

	if (s_inizialized){
		glCopyTexSubImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_Y,0,0,0,0,0,width_wps,height_wps);
	}
	else{
		glCopyTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_Y,0,GL_RGB,0,0,width_wps,height_wps,0);
	}

	// y negativi
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	gluLookAt(0,0,0,0,-1,0,1,0,0);

	glEnable(GL_TEXTURE_2D);
	draw_skybox(bmin,bmax);
	draw_palazzi();
	glDisable(GL_TEXTURE_2D);

	glBindTexture(GL_TEXTURE_CUBE_MAP,texName[TexSkyBoxYnegDin]);

	if (s_inizialized){
		glCopyTexSubImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y,0,0,0,0,0,width_wps,height_wps);
	}
	else{
		glCopyTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y,0,GL_RGB,0,0,width_wps,height_wps,0);
	}

		s_inizialized=true;

	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);
	glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);

	
}
//
//DISEGNO TUTTO
void disegna_tutto(vcg::Point3f bmin,vcg::Point3f bmax){

	if (NelTunnel==false) init_cube_map();
	else init_cube_map_tunnel();

	// draw players
	Race::PlayersIterator pi;
	int st = clock();

	for(pi = cli.players.begin(); pi != cli.players.end(); ++pi)
		{
			glPushMatrix();
				/* apply transformation coded in the frame */
				
				Frame f = ((*pi).car.frame);
				glMultMatrixf(&f[0][0]);
					
					
					glDisable(GL_TEXTURE_2D);		
					//ablito il cube mapping solo sulla macchina
					enable_cube_map();
					draw_car_model();
					//glutSolidSphere(2,100,10);
					//disabilito il cube mapping
					disable_cube_map();			
					glEnable(GL_TEXTURE_2D);
					
		
				draw_fari();
					
				glPushAttrib(GL_CURRENT_BIT);
				glColor3f(1.0,0.0,0.0);
				gl_print(0.0,0.0,(*pi).ip_address.c_str()); // write the IP address of the client
				glPopAttrib();

			glPopMatrix();
	 }



	//disegno lo skybox
	//draw_skybox(bmin,bmax);
	draw_skybox2();



	//disegno il terreno
	//draw_terrain(bmin,bmax);
	draw_terrain2(50,bmin,bmax);
	


	//headlight(vcg::Point3f(0,3.0,-3.0),vcg::Point3f(0,0,-7.0));
	
	//disegno gli assi
	glPushMatrix();
		glScalef(bmax[0]-bmin[0],bmax[1]-bmin[1],bmax[2]-bmin[2]);
		draw_axis();
	glPopMatrix();

	// disegno la strada
	draw_road();
	

	

	//disegno i tunnel
	draw_tunnel();

	//disegno i palazzi
	draw_palazzi();

	//disegno gli alberi
	draw_billboard_trees();
	



	

}
//render
/** render the scene */
void draw_scene(){
	static int cn = 0;
	static int part = 0;
	
	//calcolo il cubemaps dinamico
	//dinamic_cube_maps();

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);


	set_lights();
	


	//SDL_Delay(40);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60,width_wps/(float)height_wps,0.1,2000);
	//gluPerspective(60,width_wps/(float)height_wps,0.1,5000);
	//glOrtho(-sidex*0.5,sidex*0.5,-sidez*0.5,sidez*0.5,0.0,400);


	/* here take the bounding box of the whole circuit */
	vcg::Point3f bmin = race.circuit->bbox.min;
	vcg::Point3f bmax = race.circuit->bbox.max;

	float sidex,sidez;
	sidex = bmax[0]-bmin[0];
	sidez = bmax[2]-bmin[2];

	/* updte the state of the players as received by the server */
	cli.UpdatePlayers();

	Race::PlayersIterator fpi;
	vcg::Point3f position=vcg::Point3f(0,0,0);
	vcg::Point3f delta=vcg::Point3f(0,0,0);	
	vcg::Point3f delta_avanti=vcg::Point3f(0,0,0);

	if(cli.players.size()>0){
		fpi = cli.players.begin();		
		Frame myf=(*fpi).car.frame; //frame della macchina, ricevuto dal server
		position=myf.GetRow3(3); //la mia posizione (quarta riga frame)
		//di quanto mi sposto dietro e sopra la macchina (x calcolare il pov)
		delta=myf.GetRow3(2)*5+myf.GetRow3(1)*2.5f; //pongo il punto di vista dietro la macchina
		delta_avanti=myf.GetRow3(2)*(-3)+myf.GetRow3(1)*(1.5f);//guardo un po' piu' avanti  del centro della macchina		
		NelTunnel=IsInT(position);
	/*	if (NelTunnel==true){
			cout<<"nel tunnel, nel tunnel!!!!";}*/
	}
	
	//if (NelTunnel==false)  glEnable(GL_LIGHT0);
	//else glDisable(GL_LIGHT0);

	//attenuo la luce nel tunnel
	set_light_tunnel(NelTunnel);


	glMatrixMode(GL_MODELVIEW);

	glLoadIdentity();
	
	

	// gestione del punto di vista
	if(free_look==false){
	pov=position + delta; //pov x la lookAt
	gluLookAt(pov.X()+x,pov.Y()+y,pov.Z()+z,position.X()+delta_avanti.X(),position.Y()+delta_avanti.Y(),position.Z()+delta_avanti.Z(),0,1,0);
	}
	else{	
	//pov=free_pov;
	gluLookAt(free_pov.X(),free_pov.Y(),free_pov.Z(),free_position.X(),free_position.Y(),free_position.Z(),0,1,0); //posiziona la lookat
	}

	//PERCHE' SE LO DECOMMENTO NON FUNZIONA PIU' NULLA?

	//glPushMatrix();
	//glEnable(GL_TEXTURE_2D);
	//glBindTexture( GL_TEXTURE_2D, texName[3]);
	//glScalef(5,100,5);
	//glBegin(GL_QUADS);
	//glTexCoord2f(0,0);glVertex3f(0,0,0);
	//glTexCoord2f(1,0);glVertex3f(1,0,0);
	//glTexCoord2f(1,1);glVertex3f(1,1,0);
	//glTexCoord2f(0,1);glVertex3f(0,1,0);
	//glPopMatrix();
	//glDisable(GL_TEXTURE_2D);

	disegna_tutto(bmin,bmax);

	glLoadIdentity();

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	glPushAttrib(GL_CURRENT_BIT);
	glColor3f(1.0,0.0,0.0);
	/* print the commands */
	gl_print(-1,0.8,"accel: w");
	gl_print(-1,0.7,"retro: s");
	gl_print(-1,0.6,"left:  a");
	gl_print(-1,0.5,"right: d");
	gl_print(-1,0.4,"brake: x");
	gl_print(-1,0.3,"reset: p");
	gl_print(-1,0.2,"boost: shift");
	gl_print(-1,0.1,"motion blur: q");
	glPopAttrib();


	
	cn++;
	if(clock()-part >1000){	
		printf("VIEW fps %d\n",cn);
		part = clock();
		cn=0;
	}
}


/// callback function che gestisce l'evento "mouse mosso"
void mouse_move(SDL_Event e){
}
///
//void keydown(SDL_Event e){
void keydown(SDL_Event e){
	switch(e.key.keysym.sym){
		case SDLK_w: cli.Command(COMMAND_INCREASE_SPEED);break;
		case SDLK_s: cli.Command(COMMAND_DECREASE_SPEED);break;
		case SDLK_a: cli.Command(COMMAND_TURN_LEFT);break;
		case SDLK_d: cli.Command(COMMAND_TURN_RIGHT);break;
		case SDLK_x: cli.Command(COMMAND_BRAKE);break;
		case SDLK_p: cli.Command(COMMAND_RESTART);break;
		case SDLK_q: if(motion_on){motion_on=false;}else{motion_on=true;};break;
		case SDLK_LSHIFT: cli.Command(COMMAND_BOOST);break;
		case SDLK_RSHIFT: cli.Command(COMMAND_BOOST);break;

		case SDLK_u: x++; break;
		case SDLK_i: x--; break;
		case SDLK_j: y++; break;
		case SDLK_k: y--; break;
		case SDLK_n: z++; break;
		case SDLK_m: z--; break;
		case SDLK_r: x=0;y=0;z=0;break;
		//gestione vista libera

		case SDLK_SPACE:{
			if (free_look==false){
				free_look=true;

				free_pov.X()=0;
				free_pov.Y()=1000;
				free_pov.Z()=0;
				free_position=free_view(free_pov,free_position);
			}
				
			else
				free_look=false;
		}
	
		//traslazione del pov in vista libera
		case SDLK_KP8: {
			if (free_look==true){
						free_pov.X()=free_position.X()+passo*cos(alpha)*cos(tetha);
						free_pov.Y()=free_position.Y()+passo*cos(alpha)*sin(tetha);
						free_pov.Z()=free_position.Z()+passo*sin(alpha)*cos(tetha);
						//*free_position.X()=free_pov+norma*cos(alpha)*cos(tetha);
						//free_position.Y()=free_pov+norma*cos(alpha)*sin(tetha);
						//free_position.Z()=free_pov+norma*sin(alpha)*cos(tetha);*/
						free_position=free_view(free_pov,free_position);
						}
					   break;}
		case SDLK_KP5: {
			if (free_look==true){
						free_pov.X()=free_pov.X()-passo*cos(alpha)*cos(tetha);
					    free_pov.Y()=free_pov.Y()-passo*cos(alpha)*sin(tetha);
						free_pov.Z()=free_pov.Z()-passo*sin(alpha)*cos(tetha);
					    /*free_pov.X()=free_pov.X()+norma*cos(alpha)*cos(tetha);
						free_pov.Y()=free_pov.Y()+norma*cos(alpha)*sin(tetha);
						free_pov.Z()=free_pov.Z()+norma*sin(alpha)*cos(tetha);*/
						free_position=free_view(free_pov,free_position);
						}
					   break;}
		// gestione degli angoli per la direzione di vista libera
		case SDLK_KP9: if (free_look==true){alpha+=0.1;free_position=free_view(free_pov,free_position);}break;
		case SDLK_KP7: if (free_look==true){alpha-=0.1;free_position=free_view(free_pov,free_position);}break;					
		case SDLK_KP6: if (free_look==true){tetha+=0.1;free_position=free_view(free_pov,free_position);}break;
		case SDLK_KP4: if (free_look==true){tetha-=0.1;free_position=free_view(free_pov,free_position);}break;
		case SDLK_KP2: if (free_look==true){tetha=0;alpha=0;}break;

	

		//stampa a video la mia posizione attuale
		case SDLK_F8:{
		Race::PlayersIterator fpi = cli.players.begin();
		Frame myf=(*fpi).car.frame;
		cout<<myf.GetRow3(3).X();cout<<"; ";cout<<myf.GetRow3(3).Y();cout<<"; ";cout<<myf.GetRow3(3).Z();		 
		}
		case SDLK_F9: cout<<x;cout<<y;cout<<z;break;
	}
}
///
int launch_viewer(){
	

  // inizializzazione di SDL  
  SDL_Init( SDL_INIT_VIDEO ); 


  
  // facciamo una finestra di 500x500
  SDL_SetVideoMode(512,512,0, SDL_OPENGL|SDL_RESIZABLE|SDL_DOUBLEBUF);

 

	// inizializzamo lo stato di opengl
  init_opengl();

  //inizializzazione cube mapping (load textures)
	init_cube_map();


  // settiamo OpenGL alle dimensioni 500x500
  resize_screen(512,512);

  //carico il circuito
  race.circuit->Load(circuito);


  // Ripulisco il buffer accumulatore
  	glClearAccum(0.0,0.0,0.0,1.0);
	glClear(GL_ACCUM_BUFFER_BIT);
	

  //static int last_rendering = 0;
  int done=0;
  while (!done) { // loop gestione eventi
    
    SDL_Event e;
 
		// guarda se c' un evento. Che ci sia o no continua
    if(SDL_PollEvent(&e))
			// processo l'evento
			switch (e.type) {
				case SDL_KEYDOWN: 
					  keydown(e);
					break;
				case SDL_VIDEORESIZE:  
					// evento: a dim della finestra e' cambiata
					resize_screen(e.resize.w,e.resize.h);
					break;
	        
				case SDL_QUIT:  
					// evento: la applicazione deve uscire
					done=1;   
					break;
				}

	
	

	
	draw_scene();

	//MOTION BLUR
	if(motion_on) {
	float q = 0.3; 
	glAccum(GL_MULT, q); //impedisce al colore di saturare al bianco
	glAccum(GL_ACCUM, 1-q); //accumula il fotogramma passato
	glAccum(GL_RETURN, 1.0); // lo sovrappone al corrente
	glFlush(); 
	}
	SDL_GL_SwapBuffers(); 
	


	//PERCHE' NON FUNZIONA?
	//MOTION BLUR
	//render++;
	//if (render==1){
	//	glAccum(GL_LOAD,float(1/MAX_RENDER));
	//}
	//if ((1<render)&(render<=MAX_RENDER)){
	//	glAccum(GL_ACCUM, float(1/MAX_RENDER));
	//}
	//if (render==MAX_RENDER){
	//	glAccum(GL_RETURN, 1.0); 
	//	//glFlush();
	//	SDL_GL_SwapBuffers();
	//}
	//if (render>MAX_RENDER){
	//	render=0;
	//	glClearAccum(0.0,0.0,0.0,1.0);
	//	glClear(GL_ACCUM_BUFFER_BIT);
	//}
	//



  }
	return 0;
}

