#pragma once
/****************************************************************************
*                                                                           *
* Copyright(C) 2007                                                         *
* Pierluigi Failla                                                          *
*																			*
* http://www.zeta.xm.com													*
*                                                                           *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *   
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/

#include <GL/glew.h>
#include <SDL.h>

#include <GL/glut.h>

#include <vcg/space/point3.h>

#include "../client/game_client.h"
#include "game_viewer.h"
#include "race.h"

using namespace vcg;
using namespace std;

#define M_PI       3.14159265358979323846

//eye
vcg::Point3f eye;

//calculate distance
float dist2D(float x, float y){
	return (float)sqrt(x*x+y*y);
}
//draw cilinder
void draw_cylinder(void){
	int slices = 20;
	float step = 2*M_PI/slices;
	for(int i = 0; i < slices;++i){
		glBegin(GL_QUADS);
		glVertex3f( cos(i*step),-0.5,-sin(i*step));
		glVertex3f( cos((i+1)*step),-0.5,-sin((i+1)*step));
		glVertex3f( cos((i+1)*step), 0.5,-sin((i+1)*step));
		glVertex3f( cos(i*step), 0.5,-sin(i*step));
		glEnd();

		glBegin(GL_TRIANGLES);
		glVertex3f( cos(i*step),0.5,-sin(i*step) );
		glVertex3f( cos((i+1)*step),0.5,-sin((i+1)*step)  );
		glVertex3f( 0.0,0.5, 0.0);
		glVertex3f( cos(i*step),-0.5,-sin(i*step) );
		glVertex3f( 0.0,-0.5,0.0 );
		glVertex3f( cos((i+1)*step),-0.5,-sin((i+1)*step)  );
		glEnd();
	}
}
/*
void draw_buildings(GLuint TextureNumber)
Takes only the texture number to apply on the build
Enjoy with multiple iterator!!! :))
*/
void draw_buildings(GLuint TextureNumber){
	glEnable(GL_CULL_FACE); //Enable culling
	glCullFace(GL_BACK); //Remove back face
	glEnable (GL_TEXTURE_2D); //Enalbing texuring
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST); //Perspective correction for texture
	glBindTexture( GL_TEXTURE_2D, TextureNumber); //Bind with the texture

	Circuit::BuildingsIterator bi; //Iterator for buildings
	float h = 0.0, x1 = 0.0, y1 = 0.0, x2 = 0.0, y2 = 0.0, dist = 0.0;
	for(bi = race.circuit->buildings.begin(); bi != race.circuit->buildings.end(); ++bi){
		h = (*bi).height;
		for(int i=0; i < (*bi).footprint.size()-1;++i){
			glBegin(GL_QUADS); //Attach a quad for each face of the buildings
				glTexCoord2d(0,0);
				glVertex3f((*bi).footprint[i].P()[0],(*bi).footprint[i].P()[1],(*bi).footprint[i].P()[2]);
				x1 = (float)(*bi).footprint[i].P()[0];
				y1 = (float)(*bi).footprint[i].P()[2];
				++i;
				x2 = (float)(*bi).footprint[i].P()[0];
				y2 = (float)(*bi).footprint[i].P()[2];
				dist = dist2D(x2-x1,y2-y1);
				dist = dist/10;
				glTexCoord2d(dist,0);
				glVertex3f((*bi).footprint[i].P()[0],(*bi).footprint[i].P()[1],(*bi).footprint[i].P()[2]);
				glTexCoord2d(dist,1);
				glVertex3f((*bi).footprint[i].P()[0],(*bi).footprint[i].P()[1] + h,(*bi).footprint[i].P()[2]);
				--i;
				glTexCoord2d(0,1);
				glVertex3f((*bi).footprint[i].P()[0],(*bi).footprint[i].P()[1] + h,(*bi).footprint[i].P()[2]);
			glEnd();
		}
		glCullFace(GL_FRONT);
		int i = (*bi).footprint.size()-1;
		glBegin(GL_QUADS); //The last face of the buildings!!!
			glTexCoord2d(0,0);
			glVertex3f((*bi).footprint[i].P()[0],(*bi).footprint[i].P()[1],(*bi).footprint[i].P()[2]);
			x1 = (float)(*bi).footprint[i].P()[0];
			y1 = (float)(*bi).footprint[i].P()[2];
			//i = 0;
			x2 = (float)(*bi).footprint[0].P()[0];
			y2 = (float)(*bi).footprint[0].P()[2];
			dist = dist2D(x2-x1,y2-y1);
			dist = dist/10;
			glTexCoord2d(dist,0);
			glVertex3f((*bi).footprint[0].P()[0],(*bi).footprint[0].P()[1],(*bi).footprint[0].P()[2]);
			glTexCoord2d(dist,1);
			glVertex3f((*bi).footprint[0].P()[0],(*bi).footprint[0].P()[1] + h,(*bi).footprint[0].P()[2]);
			//i = (*bi).footprint.size()-1;
			glTexCoord2d(0,1);
			glVertex3f((*bi).footprint[i].P()[0],(*bi).footprint[i].P()[1] + h,(*bi).footprint[i].P()[2]);
		glEnd();
	}

	glDisable(GL_TEXTURE_2D);
	glDisable(GL_CULL_FACE);
	//happy ending!!!!!
}

/*
billboard
*/
void billboard_on(float camx, float camy, float camz, float posx, float posy, float posz){
	vcg::Point3f Camera(camx,camy,camz);
	vcg::Point3f WorldPosition(posx,posy,posz);

	float coseno = 0.0;

	vcg::Point3f QuadLookAt(0,0,1); //the quad look at +z
	vcg::Point3f OtoCp(Camera[0] - WorldPosition[0], 0, Camera[2] - WorldPosition[2]); //vector from object to camera projectioi on xz
	vcg::Point3f OtoC(Camera[0] - WorldPosition[0],Camera[1] - WorldPosition[1],Camera[2] - WorldPosition[2]); //vector object cam
	vcg::Point3f upAux(0,0,0); //angle tra OtoCp e QuadLookAt

	OtoCp.Normalize();
	OtoC.Normalize();
	
	//upAux serve per determinare l'angolo 
	upAux = QuadLookAt ^ OtoCp; 

	coseno = QuadLookAt * OtoCp;
	//numeric stability
	if ((coseno < 0.99990) && (coseno > -0.9999))
		glRotatef(acos(coseno)*180/M_PI,upAux[0], upAux[1], upAux[2]);

	coseno = OtoCp * OtoC;

	if ((coseno < 0.99990) && (coseno> -0.9999))
		if (OtoC[1] < 0)
			glRotatef(acos(coseno)*180/M_PI,1,0,0);	
		else
			glRotatef(acos(coseno)*180/M_PI,-1,0,0);	

}
//update point of view!!!
void update_eye(float x, float y, float z){
	eye[0] = x;
	eye[1] = y;
	eye[2] = z;
}

/*
draw tree
*/
void draw_trees(GLuint TextureNumber){
	glEnable(GL_TEXTURE_2D);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_ALPHA_TEST);
	glAlphaFunc(GL_GREATER, 0);
	glBindTexture(GL_TEXTURE_2D, TextureNumber);
	
	Circuit::TreesIterator tsi;
	for(tsi = race.circuit->trees.begin(); tsi != race.circuit->trees.end(); ++tsi){
		float height = (*tsi).height;
		glPushMatrix();

		glTranslatef((*tsi).pos[0],(*tsi).pos[1],(*tsi).pos[2]);//(*tsi).pos[1]
		
		billboard_on(eye[0],eye[1],eye[2],(*tsi).pos[0],(*tsi).pos[1],(*tsi).pos[2]);

		glBegin(GL_QUADS);
			glTexCoord2f(0,0);glVertex3f(-2.0,0.0,0.0);
			glTexCoord2f(1,0);glVertex3f( 2.0,0.0,0.0);
			glTexCoord2f(1,1);glVertex3f( 2.0,height,0.0);
			glTexCoord2f(0,1);glVertex3f(-2.0,height,0.0);
		glEnd();

		glPopMatrix();
	}

	glDisable(GL_BLEND);
    glDisable(GL_ALPHA_TEST);
	glDisable(GL_TEXTURE_2D);
}
/*
draw circuit takes the texture number to apply on the street
and texture for pole!!
*/
void draw_circuit(GLuint TextureNumber, GLuint TextureNumberPole){
	glEnable(GL_LIGHTING);
	glCullFace(GL_BACK);
	// draw circuit	
	Street::TilesIterator ti;
	vcg::Point3f p0,p1;
	glEnable (GL_TEXTURE_2D);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	glBindTexture( GL_TEXTURE_2D, TextureNumber);
	glBegin(GL_QUAD_STRIP);
	// with following code we loose the last quad
	for(ti = race.circuit->street.path.begin(); ti != race.circuit->street.path.end()-1;++ti){
		p0 = (*ti)[0].P(); p1 = (*ti)[1].P();
		glTexCoord2d(0.0,0.0);
		glVertex3f(p0[0],p0[1]+0.01,p0[2]);
		glTexCoord2d(1.0,0.0);
		glVertex3f(p1[0],p1[1]+0.01,p1[2]);
		++ti;
		p0 = (*ti)[0].P(); p1 = (*ti)[1].P();
		glTexCoord2d(0.0,1.0);
		glVertex3f(p0[0],p0[1]+0.01,p0[2]);
		glTexCoord2d(1.0,1.0);
		glVertex3f(p1[0],p1[1]+0.01,p1[2]);
		--ti;
	}
	glEnd();
	//street closure the last quad
	glCullFace(GL_BACK);
	glBegin(GL_QUADS);
		ti = race.circuit->street.path.end();
		--ti;
		p0 = (*ti)[0].P(); p1 = (*ti)[1].P();
		glTexCoord2d(1,0);
		glVertex3f(p0[0],p0[1]+0.01,p0[2]);
		glTexCoord2d(0,0);
		glVertex3f(p1[0],p1[1]+0.01,p1[2]);
		ti = race.circuit->street.path.begin();
		p0 = (*ti)[0].P(); p1 = (*ti)[1].P();
		glTexCoord2d(1,1);
		glVertex3f(p0[0],p0[1]+0.01,p0[2]);
		glTexCoord2d(0,1);
		glVertex3f(p1[0],p1[1]+0.01,p1[2]);
	glEnd();

	// draw piloni	
	glEnable(GL_TEXTURE_2D);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_ALPHA_TEST);
	glAlphaFunc(GL_GREATER, 0);
	glBindTexture(GL_TEXTURE_2D, TextureNumberPole);
	for(ti = race.circuit->street.path.begin(); ti != race.circuit->street.path.end();++ti){
		p0 = (*ti)[0].P(); p1 = (*ti)[1].P();
		glPushMatrix();

		glTranslatef(p0[0],p0[1],p0[2]);
		
		billboard_on(eye[0],eye[1],eye[2],p0[0],p0[1],p0[2]);

		glBegin(GL_QUADS);
			glNormal3f(0,0,1);
			glTexCoord2f(0,0);glVertex3f(-1.0,0.0,0.0);
			glTexCoord2f(1,0);glVertex3f( 1.0,0.0,0.0);
			glTexCoord2f(1,1);glVertex3f( 1.0,1.0,0.0);
			glTexCoord2f(0,1);glVertex3f(-1.0,1.0,0.0);
		glEnd();

		glPopMatrix();

		glPushMatrix();

		glTranslatef(p1[0],p1[1],p1[2]);
		
		billboard_on(eye[0],eye[1],eye[2],p1[0],p1[1],p1[2]);

		glBegin(GL_QUADS);
			glNormal3f(0,0,1);
			glTexCoord2f(0,0);glVertex3f(-1.0,0.0,0.0);
			glTexCoord2f(1,0);glVertex3f( 1.0,0.0,0.0);
			glTexCoord2f(1,1);glVertex3f( 1.0,1.0,0.0);
			glTexCoord2f(0,1);glVertex3f(-1.0,1.0,0.0);
		glEnd();

		glPopMatrix();
	}

	glDisable(GL_BLEND);
    glDisable(GL_ALPHA_TEST);
	glDisable(GL_TEXTURE_2D);
}
/*
Motion blurring
*/
float q = 0.91;
void motion_blurring_daemon(void){
	glAccum(GL_MULT, q);
	glAccum(GL_ACCUM, 1-q);
}

void motion_blurring(void){
	glAccum(GL_RETURN, 1.0);
	SDL_GL_SwapBuffers();
}

//float * inverse_rot_frame(Frame f){
//	float *mm;
//	mm = new float[16];
//	mm[0] = f[0][0];
//	mm[1] = f[0][4];
//	mm[2] = f[0][8];
//	mm[3] = f[0][3];
//	mm[4] = f[0][1];
//	mm[5] = f[0][5];
//	mm[6] = f[0][9];
//	mm[7] = f[0][7];
//	mm[8] = f[0][2];
//	mm[9] = f[0][6];
//	mm[10] = f[0][10];
//	mm[11] = f[0][11];
//	mm[12] = 0.0;//f[0][12];
//	mm[13] = 0.0;//f[0][13];
//	mm[14] = 0.0;//f[0][14];
//	mm[15] = 1.0;//f[0][15];
//	return mm;
//}