#include <SDL.h>

#include <GL/glew.h>
#include <GL/glut.h>
#include <GL/glu.h>
#include <vector>

// mesh definition
#include <vcg/simplex/vertexplus/base.h>
#include <vcg/simplex/faceplus/base.h>
#include <vcg/complex/trimesh/base.h>
#include <vcg/complex/trimesh/update/bounding.h>
#include <vcg/complex/trimesh/update/normal.h>
#include <wrap/io_trimesh/import.h>
#include <wrap/gl/trimesh.h>
#include <wrap/gui/trackball.h>

#include "thecar.h"

class CEdge;    // dummy prototype never used
class CFace;

using namespace std;
using namespace vcg;

class CVertex : public VertexSimp2< CVertex, CEdge, CFace, vert::Coord3f, vert::Normal3f >{};//face::WedgeTexture2f
class CFace   : public FaceSimp2<   CVertex, CEdge, CFace, face::VertexRef, face::Normal3f, face::Color4b> {};
class CMesh   : public vcg::tri::TriMesh< vector<CVertex>, vector<CFace> > {};

CMesh mesh; //car mesh
vcg::GlTrimesh<CMesh> glWrap;

char* CAR_MODEL_PATH[8] = { 
	"../../client3/models/newbeetle.ply",
	"../../client3/models/ruspa.ply",
	"../../client3/models/bus.ply",
	"../../client3/models/camioncino2.ply",
	"../../client3/models/camioncino.ply",
	"../../client3/models/schiacciasassi.ply",
	"../../client3/models/schiacciasassi2.ply",
	"../../client3/models/ruspino.ply",
};

int CAR_MODEL_INDEX = 0;
int CAR_DIR = 90;

void loadCar(){
	printf("LOAD CAR MODEL!\n");
	// Generic loading of the mesh from disk
	if(vcg::tri::io::Importer<CMesh>::Open(mesh,CAR_MODEL_PATH[CAR_MODEL_INDEX])!=0) {
		fprintf(stderr,"Error reading file %s\n",CAR_MODEL_PATH[CAR_MODEL_INDEX]);
		exit(0);
	}

	// Initialize the mesh itself
	vcg::tri::UpdateBounding<CMesh>::Box(mesh);      // update bounding box
	vcg::tri::UpdateNormals<CMesh>::PerVertexPerFace(mesh); // update Normals
		
	// Initialize the wrapper
	glWrap.m = &mesh;
	glWrap.Update();
}

void drawCar(){	
	printf("DRAW CAR!\n");

	glPushMatrix();

	glMateriali(GL_FRONT, GL_SHININESS, 55);
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_COLOR_MATERIAL);

    //glMatrixMode(GL_MODELVIEW);
	glCullFace(GL_BACK);
	
	glPushMatrix();	
		float d=mesh.bbox.Diag();
		glScale(6/d);
		glRotatef(-90,1,0,0);
		glRotatef(CAR_DIR,0,0,1);
		glTranslate(-glWrap.m->bbox.Center());	
		glWrap.Draw<vcg::GLW::DMSmooth, vcg::GLW::CMPerFace,vcg::GLW::TMPerVert> ();
	glPopMatrix();

	glDisable(GL_COLOR_MATERIAL);
	
	vcg::Point4f spotpos = vcg::Point4f ( -0.9 , 0.3 , -4 ,1.0);
	vcg::Point4f spotpos2 = vcg::Point4f ( 0.9, 0.3, -4, 1.0);
	vcg::Point4f spotdir=vcg::Point4f (0.0, 0.0, -1.0, 0.0);

	glLightfv(GL_LIGHT1,GL_POSITION,&spotpos[0]);
	glLightfv(GL_LIGHT1,GL_SPOT_DIRECTION, &spotdir[0] );
	
	glLightfv(GL_LIGHT2,GL_POSITION,&spotpos2[0]);
	glLightfv(GL_LIGHT2,GL_SPOT_DIRECTION, &spotdir[0] );

	glEnable(GL_TEXTURE_2D);

	glPopMatrix();	
}

GLuint genCar(){
	printf("COMPILE CAR!\n");
	GLuint mycar = glGenLists(1);
	glNewList(mycar, GL_COMPILE);
		drawCar();
	glEndList();
	return mycar;
}

GLuint changeCar(int index){
	printf("CHANGE CAR!\n");
	CAR_MODEL_INDEX = index;
	loadCar();
	GLuint mycar = genCar();
	return mycar;
}

GLuint updateCarDir(){
	printf("UPDATE CAR DIRECTION!\n");
	CAR_DIR += 90;
	GLuint mycar = genCar();
	return mycar;
}