/****************************************************************************
* NVMC                                                                      *
* Visual and Computer Graphics Library                                      *
*                                                                           *
* Copyright(C) 2007                                                         *
* ISTI - Italian National Research Council                                  *
*                                                                           *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.															*
*																			*
* lallopizzi giancarlo														*
*                                                                           *
****************************************************************************/

#include <SDL.h>

#include <time.h>
#include <math.h>
#include <vcg/space/point3.h>
#include <vector>
#include <GL/glut.h>
#include <GL/glu.h>

#include "game_viewer.h"
#include "race.h"
#include "myTextures.h"


using vcg::Point3f;

extern bool tball, motion_blur, shadow, luci, shadowMap;

//Se true uso skybox meno buona
bool mysbox = false;

//Dimensioni della schermata
extern int width, height;
//Il punto di vista
extern vcg::Point3f pov, pos;
//Frame della macchina
extern Frame myf;
//Posizione del sole per Project Shadow
GLfloat lightPos[] = { 0.0f, 10000.0f, 0.0f, 0.0f };
// Transformation matrix to project shadow
GLfloat shadowMat[4][4];
//Tre punti del piano del terreno
GLfloat points[3][3] = {{ 100.0f, 0.021f, 0.0f },{ 0.0f, 0.021f, 100.0f },{100.0f, 0.021f, 100.0f }};
//Nebbia
float fog_color[4] = { 0.9, 0.9, 0.85, 0.75 };
float density = 0.15f;

/** draw reference system*/
void draw_axis(){

	glPushAttrib(0xffffffff);
	glDisable(GL_LIGHTING);
	glBegin(GL_LINES);
	glColor3f(1.0,0.0,0.0);
  glVertex3f(0,0,0); //   X
  glVertex3f(10,0,0); 
  
	glColor3f(0.0,1.0,0.0);
  glVertex3f(0,0,0); //   Y
  glVertex3f(0,10,0);
  
	glColor3f(0.0,0.0,1.0);
  glVertex3f(0,0,0); //   Z
  glVertex3f(0,0,10);
  glEnd();
  glEnable(GL_LIGHTING);
 	glPopAttrib();
}


//Stampa dei comandi a video
void gl_print(float x, float y, const char *str) {
  glRasterPos2f(x, y);
  int len = (int)strlen(str);
  for(int i = 0; i < len; i++) 
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, str[i]);
}

void draw_commands(){

	/* print the commands */
	gl_print(-1,0.9,"accel: w");
	gl_print(-1,0.85,"slow:  s");
	gl_print(-1,0.8,"left:  a");
	gl_print(-1,0.75,"right: d");
	gl_print(-1,0.7,"brake: x");
	gl_print(-1,0.65,"reset position: p");
	//Miei comandi
	gl_print(-1,0.6,"mappa: m");
	if(!luci)
		gl_print(-1,0.55,"enable car light: l");
	else
		gl_print(-1,0.55,"disable car light: l");
	gl_print(-1,0.5,"reset trackball: r");
	gl_print(-1,0.45,"enable/disable skybox: F1");
	if(!tball)
		gl_print(-1,0.4,"enable trackball: F2");
	else
		gl_print(-1,0.4,"disable trackball: F2");
	gl_print(-1,0.35,"enable/disable tree billboard: F3");
	if(!shadow)
		gl_print(-1,0.3,"enable projective shadow: F4");
	else
		gl_print(-1,0.3,"disable projective shadow: F4");
	gl_print(-1,0.25,"enable/disable tunnel: F5");
	if(!motion_blur)
		gl_print(-1,0.20,"enable motion blur: F6");
	else
		gl_print(-1,0.20,"disable motion blur: F6");
	if(!shadowMap)
		gl_print(-1,0.15,"enable shadow mapping: F7");
	else
		gl_print(-1,0.15,"disable shadow mapping: F7");

}


/** callback function for viewport resize.*/
void resize_screen(int w, int h){

	width = w;
	height = h;
	glViewport(0,0,width,height);
}

void drawCircuit(){
	
	glPushMatrix();
	glPushAttrib(GL_CURRENT_BIT);

	glTranslatef(0,0.01,0);
	
	// draw circuit	
	Street::TilesIterator ti;
	int i;
	int h=0,j=0,k=0;
	glColor3f(1.0,0.0,0.0);
	glBegin(GL_QUAD_STRIP);
	for(i=0,ti = race.circuit->street.path.begin(); ti != race.circuit->street.path.end();++ti,++i){
		vcg::Point3f p0,p1;
		p0 = (*ti)[0].P(); p1 = (*ti)[1].P();
		
		if(k==0){
			h=0;j=0;
			k=1;
		} else{
			h=0;j=1;
			k=0;
		}
		//Forzo la normale
		glNormal3f(0.0,1.0,0.0);
		glTexCoord2f(h++,j);
		glVertex3f(p0[0],p0[1],p0[2]+10);
		glTexCoord2f(h,j);
		glVertex3f(p1[0],p1[1],p1[2]+10);
	}
	glEnd();

	glPopAttrib();
	glPopMatrix();
	


}

void drawTerrain(){
	
	glPushAttrib(GL_CURRENT_BIT);
	glPushMatrix();
	/* here take the bounding box of the whole circuit */
	Point3f bmin = race.circuit->bbox.min;
	Point3f bmax = race.circuit->bbox.max;

	// draw bounding box of the terrain
	glColor3f(0.5,0.5,0.5);
	glBegin(GL_QUADS);
	glNormal3f(0,1,0);
	glTexCoord2f(0,0);
	glVertex3f(bmin[0],bmin[1],bmin[2]);
	glTexCoord2f(0,100);
	glVertex3f(bmax[0],bmin[1],bmin[2]);
	glTexCoord2f(100,0);
	glVertex3f(bmax[0],bmin[1],bmax[2]);
	glTexCoord2f(100,100);
	glVertex3f(bmin[0],bmin[1],bmax[2]);
	glEnd();

	glPopAttrib();
	glPopMatrix();
	
}

void billboarding(){
	

	//Prendo la terza colonna della matrice model-view invertita (trasposta)
	float eyemv[16];
	glGetFloatv(GL_MODELVIEW_MATRIX,eyemv);
	Point3f v_eye = &eyemv[8];

	//Il quad si suppone orientato verzo zp
	Point3f v_front = Point3f(0,0,1);
	Point3f v_right = Point3f(1,0,0);

	//Trovo seno e coseno dell'angolo tra quad e direzione di vista
	float cos = v_eye*v_front;
	float sin = v_eye*v_right;

	//Costruisco la matrice che mi ruota il quad
	float rotation_matrix[16] = {cos, 0.0f, sin, 0.0f,
							0.0f, 1.0f, 0.0f, 0.0f,
							-sin, 0.0f, cos, 0.0f,
							0.0f, 0.0f, 0.0f, 1.0f};
	
	//Moltiplico il frame del albero per la rotazione calcolata
	glMultMatrixf(rotation_matrix);

	
}

//Disegna gli alberi con la tecnica del billboard
void drawTrees(){
	
	
	glPushAttrib(GL_CURRENT_BIT);
	int k=0;
	int heights[] = {3,2,1,4,7,6};//moltiplicatori "casuali" per altezza alberi

	glColor3f(0.0,0.0,0.0);
    Circuit::TreesIterator ti;
	for(ti=race.circuit->trees.begin();ti!=race.circuit->trees.end();++ti){
		glPushMatrix();

			float height = (*ti).height*heights[k%6];
			k++;
			float treePos[3]={(*ti).pos.X(),(*ti).pos.Y(),(*ti).pos.Z()};
			glTranslatef(treePos[0],treePos[1],treePos[2]);

			//Ruoto
			billboarding();
			//Disegno gli alberi
			glBegin(GL_QUADS);
				glTexCoord2f(1,1);glVertex3f(-(height), 0.0f, 0.0f);
				glTexCoord2f(0,1);glVertex3f((height), 0.0f, 0.0f);
				glTexCoord2f(0,0);glVertex3f((height), height*2,  0.0f);
				glTexCoord2f(1,0);glVertex3f(-(height), height*2,  0.0f);
			glEnd();
		glPopMatrix();
	}
	glPopAttrib();
	
	
}

void draw_unit_cube(bool cubeMap){

	float h;
	bool aux;
	
	//Se voglio fare la seconda skybox, allora devo fare met cubo con environment mapping
	if(mysbox) 
		aux = !cubeMap;
	else
		aux = cubeMap;

	//Se voglio fare la skybox
	if(aux)
		h = 0.0; //Genero un cubo tagliato a met dal piano y=0
	else 
		h = -1.0; //Genero un cubo di lato 2 centrato in 0

	//NOT skybox
	if(!cubeMap){
		// faccia 0,0,1 
		glBegin(GL_QUADS);
		glVertex3d( -1.0, h, 1.0);
		glVertex3d(  1.0, h, 1.0);
		glVertex3d(  1.0,  1.0, 1.0);
		glVertex3d( -1.0,  1.0, 1.0);
		glEnd();

		// faccia 1,0,0 
		glBegin(GL_QUADS);
		glVertex3d( 1.0, h,  1.0);
		glVertex3d( 1.0, h, -1.0);
		glVertex3d( 1.0,  1.0, -1.0);
		glVertex3d( 1.0,  1.0,  1.0);
		glEnd();

		// faccia 0,0,-1 
		glBegin(GL_QUADS);
		glVertex3d( 1.0, h, -1.0);
		glVertex3d(-1.0, h, -1.0);
		glVertex3d(-1.0,  1.0, -1.0);
		glVertex3d( 1.0,  1.0, -1.0);
		glEnd();

		// faccia -1,0,0 
		glBegin(GL_QUADS);
		glVertex3d(-1.0, h, -1.0);
		glVertex3d(-1.0, h,  1.0);
		glVertex3d(-1.0,  1.0,  1.0);
		glVertex3d(-1.0,  1.0, -1.0);
		glEnd();

		// faccia  0,1,0 
		glBegin(GL_QUADS);
		glVertex3d(-1.0,  1.0,  1.0);
		glVertex3d( 1.0,  1.0,  1.0);
		glVertex3d( 1.0,  1.0, -1.0);
		glVertex3d(-1.0,  1.0, -1.0);
		glEnd();


		// faccia  0,-1,0 
		glBegin(GL_QUADS);
		glVertex3d(-1.0, h, -1.0);
		glVertex3d( 1.0, h, -1.0);
		glVertex3d( 1.0, h,  1.0);
		glVertex3d(-1.0, h,  1.0);
		glEnd();
	}

	//Skybox
	else{
		
		glColor3f(1.0,1.0,1.0);
		// faccia 1,0,0 XP
		switchTexture(5);
		glBegin(GL_QUADS);
		glNormal3f(-1,0,0);
		glTexCoord2f(1.0,1.0); glVertex3d( 1.0, h,  1.0);
		glTexCoord2f(0.0,1.0); glVertex3d( 1.0, h, -1.0);
		glTexCoord2f(0.0,0.0); glVertex3d( 1.0,  1.0, -1.0);
		glTexCoord2f(1.0,0.0); glVertex3d( 1.0,  1.0,  1.0);
		glEnd();

		// faccia -1,0,0 XN
		switchTexture(6);
		glBegin(GL_QUADS);
		glNormal3f(1,0,0);
		glTexCoord2f(1.0,1.0); glVertex3d(-1.0, h, -1.0);
		glTexCoord2f(0.0,1.0); glVertex3d(-1.0, h,  1.0);
		glTexCoord2f(0.0,0.0); glVertex3d(-1.0,  1.0,  1.0);
		glTexCoord2f(1.0,0.0); glVertex3d(-1.0,  1.0, -1.0);
		glEnd();

		//// faccia  0,1,0 YP
		//switchTexture(7);
		//glBegin(GL_QUADS);
		//glNormal3f(0,-1,0);
		//glTexCoord2f(1.0,1.0); glVertex3d(-1.0,  1.0,  1.0);
		//glTexCoord2f(1.0,0.0); glVertex3d( 1.0,  1.0,  1.0);
		//glTexCoord2f(0.0,1.0); glVertex3d( 1.0,  1.0, -1.0);
		//glTexCoord2f(0.0,0.0); glVertex3d(-1.0,  1.0, -1.0);
		//glEnd();

		// faccia 0,0,1 ZP
		switchTexture(9);
		glBegin(GL_QUADS);
		glNormal3f(0,0,-1);
		glTexCoord2f(1.0, 1.0); glVertex3d( -1.0, h, 1.0);
		glTexCoord2f(0.0, 1.0); glVertex3d(  1.0, h, 1.0);
		glTexCoord2f(0.0, 0.0); glVertex3d(  1.0,  1.0, 1.0);
		glTexCoord2f(1.0, 0.0); glVertex3d( -1.0,  1.0, 1.0);
		glEnd();

		// faccia 0,0,-1 ZN
		switchTexture(10);
		glBegin(GL_QUADS);
		glNormal3f(0,0,1);
		glTexCoord2f(1.0,1.0); glVertex3d( 1.0, h, -1.0);
		glTexCoord2f(0.0,1.0); glVertex3d(-1.0, h, -1.0);
		glTexCoord2f(0.0,0.0);	glVertex3d(-1.0,  1.0, -1.0);
		glTexCoord2f(1.0,0.0); glVertex3d( 1.0,  1.0, -1.0);
		glEnd();

		

	}
}


void drawBuildings(bool shadow){


	//shadow = false -> disegno gli edifici
	//shadow = true  -> disegno le ombre degli edifici
 
	glPushMatrix();
	glPushAttrib(GL_CURRENT_BIT);
	
	if(shadow){
		/*float xl = 10.0;
		float yl = 10000.0;
		float zl = 0.0;

		GLfloat m[16];
		for(int i=0;i<15;i++) 
			m[i]=0.0;

		m[0]=m[5]=m[10]=1.0;
		m[7]=-1.0/yl;

		glTranslatef(xl,yl,zl);
		glMultMatrixf(m);
		glTranslatef(-xl,-yl,-zl);*/
		//Moltiplico per la matric di proiezione delle ombre
		glMultMatrixf((GLfloat *)shadowMat);
		glColor4f(0.0,0.0,0.0,1.0);
		//glTranslatef(20,10,10);

	}


	Circuit::BuildingsIterator bi; //Iterator for buildings
	float h = 0.0;
	float l = 0.0;
	float p = 0.0;
	int k=0;
	int heights[] = {3,2,17,4,60,6};//moltiplicatori "casuali" per altezza dei palazzi

	for(bi = race.circuit->buildings.begin(); bi != race.circuit->buildings.end(); ++bi){
		
		h = (*bi).height * heights[k%6]; //altezza - y
		k++;

		
		vcg::Point3f p0, p1, p2, p3, p4, p5, p6; 
		p0 = (*bi).footprint[0].P();
		p1 = (*bi).footprint[1].P();
		p2 = (*bi).footprint[2].P();
		p3 = (*bi).footprint[3].P();
		p4 = p1 - p0;//larghezza - x (1,0,0)
		p5 = p2 - p1;//profondit - z (0,0,1)
		float centro_x = p0.X() + p4.X()/2;
		float centro_z = p0.Z() + p5.Z()/2;
		float centro_y = h/2;
		
		//Le ombre le faccio con dei cubi
		if(shadow){
			if((p4.X()>0)&&(p5.Z()>0)){
				glPushMatrix();
					glTranslatef(centro_x, centro_y, centro_z);
					glScalef(p4.X()/2,h/2,p5.Z()/2);
					draw_unit_cube(false);
				glPopMatrix();
			}
		}

		//I palazzi li faccio con quad e texture
		else{

			for(int i=0; i < (*bi).footprint.size();i++){
				//per ogni punto inizio un quad
				glBegin(GL_QUADS);
				
					glTexCoord2d(0,0);
					glVertex3f((*bi).footprint[i].P()[0],(*bi).footprint[i].P()[1],(*bi).footprint[i].P()[2]);
					
					glTexCoord2d(5,0);
					glVertex3f((*bi).footprint[(i+1)%4].P()[0],(*bi).footprint[(i+1)%4].P()[1],(*bi).footprint[(i+1)%4].P()[2]);

					glTexCoord2d(5,heights[k%6]);
					glVertex3f((*bi).footprint[(i+1)%4].P()[0],(*bi).footprint[(i+1)%4].P()[1] + h,(*bi).footprint[(i+1)%4].P()[2]);
					
					glTexCoord2d(0,heights[k%6]);
					glVertex3f((*bi).footprint[i].P()[0],(*bi).footprint[i].P()[1] + h,(*bi).footprint[i].P()[2]);
				
				glEnd();
			}
			//Faccia superiore
			glBegin(GL_QUADS); 
				glVertex3f(p0.X(),p0.Y() + h,p0.Z());
				glVertex3f(p1.X(),p1.Y() + h, p1.Z());
				glVertex3f(p2.X(),p2.Y() + h,p2.Z());
				glVertex3f(p3.X(),p3.Y() + h,p3.Z());
			glEnd();
		}
	}

	glPopAttrib();
	glPopMatrix();
	
}






void drawSkyBox(){
	
	if(mysbox){
		glPushMatrix();
		
		glMatrixMode(GL_TEXTURE);
		glPushMatrix();
			glLoadIdentity();
			glScalef(1.0,0.5,1.0);
			glTranslatef(0.0,-0.25,0.0);
			

			glMatrixMode(GL_MODELVIEW);
	}
		glLoadIdentity();
		gluLookAt(pov.X(),pov.Y(),pov.Z(),pos.X(),pos.Y(),pos.Z(),0,1,0);
		glTranslatef(pov.X(),pov.Y(),pov.Z());
		
		if(mysbox) glScalef(0.5,1.0,0.5);
		
		//draw_unit_cube(true);

		draw_unit_cube(false);
		
	if(mysbox){
			glMatrixMode(GL_TEXTURE);
		glPopMatrix();

		glMatrixMode(GL_MODELVIEW);
		glPopMatrix();
	}
}



// Reduces a normal vector specified as a set of three coordinates,
// to a unit normal vector of length 1.
void ReduceToUnit(float vector[3]){
	float length;
	// Calculate the length of the vector
	length = (float)sqrt((vector[0]*vector[0]) + (vector[1]*vector[1]) + (vector[2]*vector[2]));
	// Keep the program from blowing up by providing an acceptable
	// value for vectors whose length may be calculated too close to
	//zero.
	if(length == 0.0f)
		length = 1.0f;
	// Dividing each element by the length will result in a
	// unit normal vector.
	vector[0] /= length;
	vector[1] /= length;
	vector[2] /= length;
}
// Points p1, p2, & p3 specified in counterclockwise order
void calcNormal(float v[3][3], float out[3]){

	float v1[3],v2[3];
	static const int x = 0;
	static const int y = 1;
	static const int z = 2;
	// Calculate two vectors from the three points
	v1[x] = v[0][x] - v[1][x];
	v1[y] = v[0][y] - v[1][y];
	v1[z] = v[0][z] - v[1][z];
	v2[x] = v[1][x] - v[2][x];
	v2[y] = v[1][y] - v[2][y];
	v2[z] = v[1][z] - v[2][z];
	// Take the cross product of the two vectors to get
	// the normal vector which will be stored in out[]
	out[x] = v1[y]*v2[z] - v1[z]*v2[y];
	out[y] = v1[z]*v2[x] - v1[x]*v2[z];
	out[z] = v1[x]*v2[y] - v1[y]*v2[x];
	// Normalize the vector (shorten length to one)
	ReduceToUnit(out);
}
// Creates a shadow projection matrix out of the plane equation
// coefficients and the position of the light. The return value is stored
// in destMat[][]
void MakeShadowMatrix(){

	//Coefficienti del piano
	GLfloat planeCoeff[4];
	
	GLfloat dot;
	
	// Find the plane equation coefficients
	// Find the first three coefficients the same way we
	// find a normal.
	
	calcNormal(points,planeCoeff);

	// Find the last coefficient by back substitutions
	planeCoeff[3] = - (	(planeCoeff[0]*points[2][0]) +	(planeCoeff[1]*points[2][1]) +	(planeCoeff[2]*points[2][2]));

	// Dot product of plane and light position
	dot = planeCoeff[0] * lightPos[0] +	planeCoeff[1] * lightPos[1] +	planeCoeff[2] * lightPos[2] +	planeCoeff[3] * lightPos[3];

	// Now do the projection
	// First column
	shadowMat[0][0] = dot - lightPos[0] * planeCoeff[0];
	shadowMat[1][0] = 0.0f - lightPos[0] * planeCoeff[1];
	shadowMat[2][0] = 0.0f - lightPos[0] * planeCoeff[2];
	shadowMat[3][0] = 0.0f - lightPos[0] * planeCoeff[3];
	// Second column
	shadowMat[0][1] = 0.0f - lightPos[1] * planeCoeff[0];
	shadowMat[1][1] = dot - lightPos[1] * planeCoeff[1];
	shadowMat[2][1] = 0.0f - lightPos[1] * planeCoeff[2];
	shadowMat[3][1] = 0.0f - lightPos[1] * planeCoeff[3];
	// Third Column
	shadowMat[0][2] = 0.0f - lightPos[2] * planeCoeff[0];
	shadowMat[1][2] = 0.0f - lightPos[2] * planeCoeff[1];
	shadowMat[2][2] = dot - lightPos[2] * planeCoeff[2];
	shadowMat[3][2] = 0.0f - lightPos[2] * planeCoeff[3];
	// Fourth Column
	shadowMat[0][3] = 0.0f - lightPos[3] * planeCoeff[0];
	shadowMat[1][3] = 0.0f - lightPos[3] * planeCoeff[1];
	shadowMat[2][3] = 0.0f - lightPos[3] * planeCoeff[2];
	shadowMat[3][3] = dot - lightPos[3] * planeCoeff[3];
}
void setFog(){
	glEnable(GL_FOG);
	glFogf(GL_FOG_MODE, GL_LINEAR);
	glFogfv(GL_FOG_COLOR, fog_color);
	glFogf(GL_FOG_DENSITY, density);
	glFogf(GL_FOG_START, 100.0f);
	glFogf(GL_FOG_END, 1000.0f);
	glHint(GL_FOG_HINT, GL_NICEST);
}

void drawTunnel(){
	//glLoadIdentity();
	// draw tunnels	
	
	Street::TilesIterator ti;
	Street::TunnelsIterator  tni;
	glColor3f(1.0,1.0,0.0);
	glPolygonMode(GL_FRONT,GL_FILL);
	int i;
	for(i=0,tni = race.circuit->street.tunnels.begin(); tni != race.circuit->street.tunnels.end();++tni,++i){
		glBegin(GL_QUAD_STRIP);
		for(i=0,ti = (*tni).path.begin(); ti != (*tni).path.end();++ti,++i)	{
			vcg::Point3f p0,p1;
			p0 = (*ti)[0].P(); p1 = (*ti)[1].P();
			glVertex3f(p0[0],p0[1]+2.2,p0[2]);
			glVertex3f(p1[0],p1[1]+2.2,p1[2]);
		}
		glEnd();
	
	}
}
